
c----------------------------------------------------------------------
c ## Shapiro filter, from Shapiro(1969), J Geoph and Space Ph
c----------------------------------------------------------------------
      subroutine shapd1(b1,bout,ndim,norder,scheme)

c      implicit undefined (a-z)
      integer nxm, nym, nmax
      parameter (nxm = 300, nym = 300, nmax=1000)

      integer ndim, norder, scheme
      real*8  b1(nmax),bn(nmax),bout(nmax),fourk(20),nordsq
c  Local variables
      integer      i, i1, i2, k, kit, nord2, ndm1

c      REAL      b1(ndim),bn(ndim),bout(ndim),fourk(20)
c  changed ndim to nxm here to conform to aux and aux2 which we call
c  b1(ndim)            vector to be filtered
c  ndim                dimension of b1,bout   (ndim<200)
c  norder              order of the shapiro filter
c  scheme              flag for the type of boundary scheme to be used
c                      if scheme=
c                          1       no change at wall - constant order
c                          2       smoothing at wall - constant order
c                          3       no change at wall - reduced order
c                          4       smoothing at wall - reduced order
c                          5       periodic          - constant order
c			   6	   no change at wall - no overwriting
c                                  (1=ndm1 and 2=ndim)
c                      any other scheme is rejected
c  fourk  (out)              scaling factors
c
c  output:
c
c  bout(ndim)          correction field (not filtered field)



      nord2 = norder/2
      ndm1 = ndim-1
      nordsq= (0.5d0)**(norder)
c  scheme 1:  constant order and no change at wall

      if (scheme.eq.1) then
        do 120 k=1,nord2
          if(k.ne.nord2)  then
            bn(1) = 2.*(b1(1) - b1(2))
            bn(ndim) = 2.*(b1(ndim)-b1(ndm1))
          else
            bn(1) = 0.
            bn(ndim) = 0.
          end if
          do 100 i=2,ndm1
            bn(i) =(2*b1(i)-b1(i-1)-b1(i+1))
  100     continue
          do 110 i=1,ndim
            b1(i)=bn(i)
  110     continue
  120   continue
        do 130 i=1,ndim
          bout(i) = bn(i)*nordsq
c fourk(nord2)
  130   continue
        return

c  scheme 2:  constant order, smoothed at edges

      else if (scheme.eq.2) then
        do 160 k=1,nord2
          bn(1) = 2.*(b1(1) - b1(2))
          bn(ndim) = 2.*(b1(ndim)-b1(ndm1))
          do 140 i=2,ndm1
            bn(i) =(2*b1(i)-b1(i-1)-b1(i+1))
  140     continue
          do 150 i=1,ndim
            b1(i)=bn(i)
  150     continue
  160   continue
        do 170 i=1,ndim
          bout(i) = bn(i)*nordsq
c fourk(nord2)
  170   continue
        return

c  scheme 3:  reduced order and no change at wall

      else if (scheme.eq.3) then
        do 210 k=1,nord2
          kit = k
          i1 = k
          i2 = ndim-k+1
          if(k.eq.1) then
            do 180 i=2,ndm1
              bn(i) = 2*b1(i)-b1(i-1)-b1(i+1)
  180       continue
            bn(1) = 2.*(b1(1) - b1(2))
            bn(ndim) = 2.*(b1(ndim)-b1(ndm1))
          else
            do 190 i=i1,i2
              bn(i) = 2*b1(i)-b1(i-1)-b1(i+1)
  190       continue
          end if
          bout(i1) = bn(i1)*(.5d0)**(2.*kit)
c fourk(kit)
          bout(i2) = bn(i2)*(.5d0)**(2.*kit)
c fourk(kit)
          do 200 i=1,ndim
            b1(i) = bn(i)
  200     continue
  210   continue
        do 220 i=i1,i2
          bout(i) = bn(i)*(.5d0)**(2.*kit)
c fourk(kit)
  220   continue
        bout(1) = 0.
        bout(ndim) = 0.
        return

c  scheme 4:  reduced order, smoothed at edges

      else if (scheme.eq.4) then
        do 260 k=1,nord2
          kit = k
          i1 = k
          i2 = ndim-k+1
          if(k.eq.1) then
            do 230 i=2,ndm1
              bn(i) = 2*b1(i)-b1(i-1)-b1(i+1)
  230       continue
            bn(1) = 2.*(b1(1) - b1(2))
            bn(ndim) = 2.*(b1(ndim)-b1(ndm1))
          else
            do 240 i=i1,i2
              bn(i) = 2*b1(i)-b1(i-1)-b1(i+1)
  240       continue
          end if
          bout(i1) = bn(i1)*(.5d0)**(2.*kit)
c fourk(kit)
          bout(i2) = bn(i2)*(.5d0)**(2.*kit)
c fourk(kit)
          do 250 i=1,ndim
            b1(i) = bn(i)
  250     continue
  260   continue
        do 270 i=i1,i2
          bout(i) = bn(i)*(.5d0)**(2.*kit)
c fourk(kit)
  270   continue
        return

c  scheme 5:  constant order, periodic

      else if (scheme.eq.5) then
        do 300  k=1,nord2
          do 280  i=2,ndm1
            bn(i) = (2*b1(i)-b1(i-1)-b1(i+1))
  280     continue
          do 290  i=2,ndm1
            b1(i) = bn(i)
  290     continue
          b1(1) = b1(ndm1)
          b1(ndim) = b1(2)
  300   continue
        do 310  i=1,ndim
          bout(i) = b1(i)*nordsq
c fourk(nord2)
  310   continue
        return

c  scheme 6:  constant order and no change at wall (without overwriting)

      else if (scheme.eq.6) then
        do 520 k=1,nord2
            bn(1) = 0.d0
            bn(ndim) = 0.d0
          do 500 i=2,ndm1
            bn(i) =(2*b1(i)-b1(i-1)-b1(i+1))
  500     continue
          do 510 i=1,ndim
            b1(i)=bn(i)
  510     continue
  520   continue
        do 530 i=1,ndim
          bout(i) = bn(i)*nordsq

  530   continue
        return

      end if
      end


