c23456789012345678901234567890123456789012345678901234567890123456789012
c-----------------------------------------------------------------------|
c|	WINC - Wave Induced Nearshore Circulation 			|
c|	Standard Version of SHORECIRC					|
c|                                                                      |
c|	Version 2.0 Linked to NOPP Master program                       |
c|									|
c|	Ib A Svendsen, Kevin A Haas and Qun Zhao			|
c|									|
c|	NOTICE:								|
c|	The record_length in the direct write to files is as:		|
c|	- CRAY : recl=1*ny						|
c|	- SGI  : recl=1*ny						|
c|	- Linux: recl=4*ny 						|
c|	- Sun  : recl=4*ny 						|
c|									|
c|	LAST MODIFICATION:						|
c|	- April 30, 2003						|
c|	Kevin A Haas 							|
c-----------------------------------------------------------------------|

c #####################################################################
c MAIN PROGRAM
      subroutine CircModule()
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'
      include 'pass.h'

c--- Local variables -------------------------------------------------
c      integer i

c=====================================================================



      if (Master_Start.eq.1) then

       call constantvals
       call inputvals
         depthmax=3.001d0     
       call numerical 
       call bathymetry
       call numerical 
       call init_output
c      write(*,901) 'Load stresses? (0=no, 1=yes) : '
c      read(*,*) i
c      if (i.eq.0) stop
       call refract_longwaves
       call shortwave_driver
       call friction_factor
c      write(*,901) 'Start current model? (0=no, 1=yes) : '
c      read(*,*) i
c      if (i.eq.0) stop
       call initialsteps
       call viscosity
       call wave_forcing
      else if (Master_Start.eq.-1) then
       call bathymetry
       call shortwave_driver
       call viscosity
       call wave_forcing
        call time_model
c      write(*,902) 'end'
      end if


901   format(/A,$)
902   format(/A/)

      return
      end


c #####################################################################
c CONSTANT VALUES
      subroutine constantvals
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'

c=====================================================================

      pi=4.d0*datan2(1.d0,1.d0)
      g=9.81d0
      rho=1015.d0
      rhod=1.d0/rho
c initialize the total number of timesteps
      ts=0

      return
      end


c #####################################################################
C INPUT VALUES
      subroutine inputvals
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'
      include 'pass.h'

c--- Local variables -------------------------------------------------
      character*35 flestr

      namelist
     .        /boundary/ibc1,ibc2,ibc3,ibc4
     .        /grids/dx,dy,nx,ny,nxwall,httide
     .        /short_wave/period,wc,amerge
c     .        /filter/nxorder,nyorder,nfilter1,nfilter2,iqxfilter,
c     .                iqyfilter,izxfilter,izyfilter
     .        /control/cr,ntime,interval,kold_start,depthmin,delay
     .        /sensor/jump,xsensor, ysensor, yprofile
     .        /outputSC/tavgout,momout,dispout
     .        /physics/fcw,vtshear,mdiss,Cs,wind_vel,wind_dir,disp3d

c=====================================================================
c--- default initializations
      idel=1.d0
      sperre=1.d0
      interval=400
      kold_start=1
      cr=0.5d0
      ntime=201
      fcw = 0.01d0
      vtshear = 0.05d0
      mdiss = 0.2d0
      jump=10
c--- read from namelist fields in file "input_winc.dat"
c ## 'period' is short wave period
c ##  koldstart : 1=yes 0=no 



      flestr= 'input_winc.dat'
      open (unit=11, file=flestr, status='old')
      read(11,nml=boundary)
      read(11,nml=grids)
      read(11,nml=short_wave)
c      read(11,nml=filter)
      read(11,nml=control)
      read(11,nml=sensor)
      read(11,nml=outputSC)
      read(11,nml=physics)

      close(11)
c  Use master defined coordinates KAH 4/30/03

      nx=Nx_Circ
      ny=Ny_Circ

c---  compute the inverse of the delay
      delay = 1.d0/delay

c--- boundary conditions
      iperx=0
      ipery=0
      if (ibc1.eq.5 .and. ibc2.eq.5) iperx=1
      if (ibc3.eq.5 .and. ibc4.eq.5) ipery=1

c--- long wave parameters
c ## alpha(1) corresponds to the angle of the incoming long-wave
c ## (if it's generated) and affects the CHARACTERISTICS @ x=0 bound.
      alpha(1)=0.d0
      alpha(1) = alpha(1)/180.d0*pi
      ampli=0.d0

c--- write control variables on screen  
      write(6,*)'nx,ny,nxwall,httide:   ',nx,ny,nxwall,httide
      write(6,901) 'ibcs: ', ibc1,ibc2,ibc3,ibc4
      write(6,902) 'Periodicy in x & y :',iperx,ipery
      write(6,903) 'fcw, vtshear, mdiss, Cs :',fcw,vtshear,mdiss,Cs
c      write(6,*) 'iqxfilter, izxfilter,  nxorder, iqyfilter, izyfilter,    
c     . nyorder,  nfilter1, nfilter2'
c      write(6,902) iqxfilter,izxfilter,nxorder,iqyfilter,izyfilter,
c     .  nyorder,nfilter1,nfilter2      


901   format(1X,A,4I4)
902   format(1X,A,2I4)
903   format(1X,A,4F12.6)

      return
      end


c #####################################################################
c BATHYMETRY
      subroutine bathymetry
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'
      include 'pass.h'
c--- Local variables -------------------------------------------------
      integer ncode,nx1,ny1,nxflag,xcor,ycor,err
      real*8 fake(nxm,nym),fake1(nxm,nym),fake2(nxm,nym),
     . xc(nym),temp,temp1,temp2

c=====================================================================

c      write(*,*) 'Update bathymetry'
c ## Initialize dummy variables <- Necessary b/c of call to DERIV !
c      do j=1,ny
c         do i=1,nx
c	    fake(i,j) = 0.d0
c	    fake1(i,j) = 0.d0
c	    fake2(i,j) = 0.d0
c            ht(i,j)=9999
c         end do
c      end do

c ##  Read the bathymetry file bath.dat 
c     Provides an error check 
ckk 9/14/99

c	open(unit=19,file='bath.dat', status='old')
c	do j=1,ny
c	  do i=1,nx
c	    read(19,*)temp1,temp2,temp
c	    xcor=nint(temp1/dx+1)
c            ycor=nint(temp2/dy+1)
c            if (nint(10000*x(xcor)).ne.nint(10000*temp1).or.
c     .           nint(10000*y(ycor)).ne.nint(10000*temp2)) then
c               write(*,*)'x=',temp1,' y=',temp2,' not on grid'
c            else if (ht(xcor,ycor).ne.9999) then
c               write(*,*)'x=',temp1,' y=',temp2,' duplicated'
c            else
c               ht(xcor,ycor)=temp+httide
c            end if
c          end do
c	end do
c        err=0
c      do j=1,ny
c         do i=1,nx
c            if (ht(i,j).eq.9999) then
c            write(*,*)'x=',x(i),' y=',y(j),' missing'
c            err=1
c            end if
c         end do
c      end do

c      if (err.eq.1) stop

c      if (ibc2.eq.4) then
c         do j = 1,ny
c            nxa(j)=nxwall
c         end do
c      end if

c      if (ibc2.eq.6) then
c         do j = 1,ny
c         nxa(j)=nx
c          do i = 1,nx
cc-##---- shoreline position
c            if ((ht(i,j)).le.depthmin) then
c               if ((ht(i-1,j)).ge.depthmin) nxa(j)=i-1
c            end if
c          end do
c         end do
c      end if

c      write(6,*) 'nxa(j)'
c      write(6,*) (nxa(j),j=1,ny)

c<===================added sep.7,1999 qun ===============>
c hs:still water level, hx:bottom slope
c      hs=3.0
c      hx=1.0/20.
c      do j=1,ny
c        do i=1,nx
c        ht(i,j)=hs-float(i-1)*dx*hx
c        end do
c      end do
      
c<===================added sep.7,1999 qun ===============>

c ## Define bathymetry for a barred beach (such as Duck)

c--- bed profile
c      do i = 1,nx
c	 if (x(i) .le. 365.) then
c            do j=1,ny
c	       ht(i,j) = 0.55d0*(370.d0-x(i))**0.35d0
c            end do
c	 else
c            do j=1,ny
c	       ht(i,j) = 0.55d0*5.d0**0.35d0 -0.06d0*(x(i)-365)
c            end do
c	 endif
c      end do

c--- crest position and 'bed profile'+'bar'
c      do j=1,ny
c         xc(j) = 0.65d0*400.d0
c      end do
c      do j=1,ny
c         do i=1,nx
c            ht(i,j) = ht(i,j) - 1.d0/150.d0*(xc(j)+250.d0-x(i))
c     .		*dexp(-15.d0*(x(i)-xc(j))**2.d0/(xc(j)*xc(j)))
c         end do
c      end do

      if (ibc2.eq.4) then
         do j = 1,ny
            nxa(j)=nxwall
         end do
      end if
c  Depth is given by master KAH 4/30/03
      do j=1,ny
         do i=1,nx
            ht(i,j)=Depth_Circ(i,j)
         end do
      end do

c          write(*,*)'SCdepth',(ht(i,1),i=1,nx)

      if (ibc2.eq.6) then
         do j = 1,ny
         nxa(j)=nx
          do i = 1,nx
cc-##---- shoreline position
            if ((ht(i,j)).le.depthmin) then
               if ((ht(i-1,j)).ge.depthmin) nxa(j)=i-1
            end if
          end do
         end do
      end if

c      Nx_wave=nxa(1)
      Nx_sedi=nxa(1)

      
c ## find maximum water depth for evaluation of Courant no.
      depthmax=0.1d0
      do j=1,ny
         do i=1,nxa(j)
c            hto(i,j) = ht(i,j)
            depthmax=DMAX1(depthmax,ht(i,j))
c	if(kold_start.eq.1) then
c	  zetan(i,j)=0.d0
c	  qxn(i,j)=0.d0
c	  qyn(i,j)=0.d0
c	  qwx(i,j)=0.d0
c	  qwy(i,j)=0.d0
c	end if
c         if (ht(i,j).le.depthmin.and.ibc2.ne.6) ht(i,j)=depthmin
         end do
      end do

c      write(*,902) ' depthmax =',depthmax



c ## Write bathymetry for REFDIF 


      open(31,file='fort.311',access='direct',recl=4*ny)
      do i=1,nx
	write(31,rec=i) (sngl(ht(i,j)),j=1,ny)
      end do
99    format(200f10.4)
      close(31)


c ## Calculate depth gradients


      ncode = 3
      nxflag = 1
      call deriv(ht,fake,fake,dhodxn,dhodyn,fake1,fake2,
     .           ncode,nxflag)

c	do j=1,ny
c	do i=1,nx
c	cdhdx(i,j)=dhodxn(i,j)
c	cdhdy(i,j)=dhodyn(i,j)
c	end	do
c	end	do

      open(29,file='fort.312',access='direct',recl=4*ny)
      open(30,file='fort.313',access='direct',recl=4*ny)
      do i=1,nx
	write(29,rec=i) (sngl(dhodxn(i,j)),j=1,ny)
	write(30,rec=i) (sngl(dhodyn(i,j)),j=1,ny)
      end do
      close(29)
      close(30)


901   format(/A)
902   format(/A,F20.4)
      return
      end


c #####################################################################
c NUMERICAL SCHEME AND GRID DEFINITION
      subroutine numerical
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'
      include 'pass.h'

c--- Local variables -------------------------------------------------
c      integer i,j
      real*8 step

c=====================================================================

      kounter=0
      step = dx*dy / DSQRT(dx*dx+dy*dy)
      dt = cr*step/DSQRT(g*depthmax)
c      write(*,*)'dt = ',dt,T_Interval_CallCirc
c Determine time step based on Master program KAH 4/30/03

c     If interval is less than dt then set dt=interval
      if (dt.gt.(N_Interval_CallCirc*Master_dt)) then
         dt=N_Interval_CallCirc*Master_dt
         ntime=1
c     If interval is interger of dt 
      elseif (mod(N_Interval_CallCirc*Master_dt,dt).eq.0) then
         dt=dt
         ntime=N_Interval_CallCirc*Master_dt/dt
c     Otherwise calculate dt
      else
         ntime=(N_Interval_CallCirc*Master_dt)/dt
     &    -mod(N_Interval_CallCirc*Master_dt,dt)+1
c         dt=cnc*cdt/ntime
         dt=(N_Interval_CallCirc*Master_dt)/ntime
      end if

      
      write(6,903) ' nx , ny , ntime',nx,ny,ntime
      write(6,902) ' dx , dy , dt',dx,dy,dt

c     Check to make sure x and y match between SC and Master
c     KAH 4/30/03
      x(1)=0.d0
      do i = 2,nx
        x(i) = x(1) + DBLE(i-1)*dx
        if (x(i).ne.X_Circ(i,1)) then
           write(*,*)'Master grid does not match SC grid'
           stop
        end if
      end do

      y(1)=0.d0
      do j = 2,ny
        y(j) = y(1) + DBLE(j-1)*dy
        if (y(j).ne.Y_Circ(1,j)) then
           write(*,*)'Master grid does not match SC grid'
           stop
        end if
      end do
      
      

901   format(/A,$)
903   format(A,5x,3I12)
902   format(A,5x,3F12.6)

      return 
      end


c #####################################################################
c REFRACT LONG WAVES
      subroutine refract_longwaves
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'

c--- Local variables -------------------------------------------------
      integer i,j
      real*8 cp
c=====================================================================

      wlec = 50.d0
      cp = DSQRT(g*depthmax)
      sigma = 2.d0*pi/wlec*cp
      j=ny/2
      do i=1,3
        alpha(i)=dasin(DSQRT(g*ht(i,j))*sin(alpha(1))/DSQRT(g*ht(1,j)))
        k(i)=sigma/DSQRT(g*ht(i,j))
      end do

      return 
      end


c #####################################################################
c SHORTWAVE DRIVER
      subroutine shortwave_driver
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'
      include 'pass.h'

c--- Local variables -------------------------------------------------
      integer  ncode, ii, nxflag, index,iflag,merge
      real*8 Sxx(nxm,nym), Sxy(nxm,nym), Syy(nxm,nym),
     . fake(nxm,nym),wavenumber, A(nxm),Afake(nxm),
     . capG,ursell,aux,qwrol(nxm),smrol(nxm),
     . xfake(nxm),y2(nxm),rol(nxm)
 	real*8 dr(nxm,nym),ur(nxm,nym),avgdhdx,roloff,
     . vr(nxm,nym)

c  Read waves from master program


      do j=1,ny
      do i=1,nx
	 H(i,j)=dble(Intp_Height_Circ(i,j))
         sxx(i,j)=dble(Intp_Sxx_Circ(i,j))
         sxy(i,j)=dble(Intp_Sxy_Circ(i,j))
         syy(i,j)=dble(Intp_Syy_Circ(i,j))
         qwx(i,j)=dble(Intp_MassFluxU_Circ(i,j))
         qwy(i,j)=dble(Intp_MassFluxV_Circ(i,j))
         u0(i,j)=dble(Intp_ubott_Circ(i,j)+.000001)
	 c_sw(i,j)=dble(Intp_C_Circ(i,j))
	 theta(i,j)=dble(Intp_theta_Circ(i,j)*pi/180d0)
         wavek(i,j)=dble(Intp_WaveNum_Circ(i,j))
         dissipation(i,j)=dble(Intp_Diss_Circ(i,j))
         ibrk(i,j)=Intp_ibrk_Circ(i,j)

c	 H(i,j)=dble(Pass_Height(i,j))
c         sxx(i,j)=dble(Pass_Sxx(i,j))
c         sxy(i,j)=dble(Pass_Sxy(i,j))
c         syy(i,j)=dble(Pass_Syy(i,j))
c         qwx(i,j)=dble(Pass_MassFluxU(i,j))
c         qwy(i,j)=dble(Pass_MassFluxV(i,j))
c         u0(i,j)=dble(Pass_ubott(i,j)+.000001)
c	 c_sw(i,j)=dble(Pass_C(i,j))
c	 theta(i,j)=dble(Pass_theta(i,j)*pi/180d0)
c         wavek(i,j)=dble(Pass_WaveNum(i,j))
c         dissipation(i,j)=dble(Pass_Diss(i,j))
c         ibrk(i,j)=Pass_ibrk(i,j)

	end do
      end do

c	print*,'yes'

c	open(35,file='tmp1.out')
c	do j=1,ny
c	write(35,134)(ibrk(i,j),i=1,nx)
c	enddo
c	close(35)

c	open(35,file='tmp2.out')
c	do j=1,ny
c	write(35,134)(Intp_ibrk_Circ(i,j),i=1,nx)
c	enddo
c	close(35)
c134	format(200I2)
	
c	pause

      if(ibc3.eq.5.and.ibc4.eq.5) then
      do i=1,nx
         H(i,1) = 0.25d0*H(i,ny-1)+0.25d0*H(i,ny)
     .             +0.25d0*H(i,1)+0.25d0*H(i,2)
         H(i,ny) = H(i,1)
         c_sw(i,1) = 0.25d0*c_sw(i,ny-1)+0.25d0*c_sw(i,ny)
     .             +0.25d0*c_sw(i,1)+0.25d0*c_sw(i,2)
         c_sw(i,ny) = c_sw(i,1)
         theta(i,1) = 0.25d0*theta(i,ny-1)+0.25d0*theta(i,ny)
     .             +0.25d0*theta(i,1)+0.25d0*theta(i,2)
         theta(i,ny) = theta(i,1)
         sxx(i,1) = 0.25d0*sxx(i,ny-1)+0.25d0*sxx(i,ny)
     .             +0.25d0*sxx(i,1)+0.25d0*sxx(i,2)
         sxx(i,ny) = sxx(i,1)
         sxy(i,1) = 0.25d0*sxy(i,ny-1)+0.25d0*sxy(i,ny)
     .             +0.25d0*sxy(i,1)+0.25d0*sxy(i,2)
         sxy(i,ny) = sxy(i,1)
         syy(i,1) = 0.25d0*syy(i,ny-1)+0.25d0*syy(i,ny)
     .             +0.25d0*syy(i,1)+0.25d0*syy(i,2)
         syy(i,ny) = syy(i,1)
      end do
      end if


c---	Calculate sin(theta) and cos(theta)  to be used globally
      do j=1,ny
	do i=1,nx
          sinth(i,j)= dsin(theta(i,j))
          costh(i,j)= dcos(theta(i,j))
	end do
      end do

      
c-- 3-pt average along the x-dir (for each j)
      call average_3ptx(nx,ny,Qw)
      call average_3ptx(nx,ny,sxx)
      call average_3ptx(nx,ny,sxy)
      call average_3ptx(nx,ny,syy)
c-- 3-pt average along the y-dir (for each i)
      if(ibc3.eq.5.and.ibc4.eq.5) then
      call average_5pty(nx,ny,sxx)
      call average_5pty(nx,ny,sxy)
      call average_5pty(nx,ny,syy)
      call average_5pty(nx,ny,Qw)
      else
      call average_5ptyold(nx,ny,sxx)
      call average_5ptyold(nx,ny,sxy)
      call average_5ptyold(nx,ny,syy)
      call average_5ptyold(nx,ny,Qw)
      end if

c ## write out Qw values -------------------------------------------
      open(14,file='fort.326',access='direct',recl=4*ny)
      open(15,file='fort.327',access='direct',recl=4*ny)
      do i=1,nx
	write(14,rec=i) (sngl(qwx(i,j)),j=1,ny)
	write(15,rec=i) (sngl(qwy(i,j)),j=1,ny)
      end do
      close(14)
      close(15)


c ## compute radiation stresses' derivatives   -----------------------

c      write(6,*) 'end input ; calculate forcing'
      ncode=3
      nxflag=1
      call deriv(Sxy,Sxx,Syy,dSxydxn,dSxydyn,dSxxdxn,dSyydyn,ncode,
     .		 nxflag)

      do j=1,ny
        do i=1,nx
           dSxxdxn(i,j)=dSxxdxn(i,j)*rhod
           dSxydxn(i,j)=dSxydxn(i,j)*rhod
           dSxydyn(i,j)=dSxydyn(i,j)*rhod
           dSyydyn(i,j)=dSyydyn(i,j)*rhod
	end do
      end do

c--- Smooth the forcing if necessary
      call average_3ptx(nx,ny,dSxxdxn)
      call average_3ptx(nx,ny,dSxydxn)
      call average_3ptx(nx,ny,dSxydyn)
      call average_3ptx(nx,ny,dSyydyn)
      if(ibc3.eq.5.and.ibc4.eq.5) then
      call average_5pty(nx,ny,dSxxdxn)
      call average_5pty(nx,ny,dSxydxn)
      call average_5pty(nx,ny,dSxydyn)
      call average_5pty(nx,ny,dSyydyn)
      else
      call average_5ptyold(nx,ny,dSxxdxn)
      call average_5ptyold(nx,ny,dSxydxn)
      call average_5ptyold(nx,ny,dSxydyn)
      call average_5ptyold(nx,ny,dSyydyn)
      end if


     
c ## write output variables and continue

c      if (mod(ts-1,interval).eq.0) then
c      write(6,*) 'Write out files with input values'
      open(14,file='fort.314',access='direct',recl=4*ny)
      open(15,file='fort.318',access='direct',recl=4*ny)
      open(16,file='fort.323',access='direct',recl=4*ny)
      open(17,file='fort.321',access='direct',recl=4*ny)
      open(18,file='fort.322',access='direct',recl=4*ny)
      open(19,file='fort.324',access='direct',recl=4*ny)
      open(25,file='fort.325',access='direct',recl=4*ny)
      open(26,file='fort.319',access='direct',recl=4*ny)
      open(27,file='fort.315',access='direct',recl=4*ny)
      open(28,file='fort.320',access='direct',recl=4*ny)
      open(29,file='fort.316',access='direct',recl=4*ny)

      do i=1,nx

	 write(14,rec=i) (sngl(H(i,j)),j=1,ny)
	 write(15,rec=i) (sngl(Sxx(i,j)),j=1,ny)
	 write(16,rec=i) (sngl(dSxydyn(i,j)),j=1,ny)
	 write(17,rec=i) (sngl(dSxxdxn(i,j)),j=1,ny)
	 write(18,rec=i) (sngl(dSxydxn(i,j)),j=1,ny)
	 write(19,rec=i) (sngl(dSyydyn(i,j)),j=1,ny)
	 write(25,rec=i) (sngl(u0(i,j)),j=1,ny)
	 write(26,rec=i) (sngl(Sxy(i,j)),j=1,ny)
	 write(27,rec=i) (sngl(theta(i,j)/pi*180.d0),j=1,ny)
	 write(28,rec=i) (sngl(Syy(i,j)),j=1,ny)
	 write(29,rec=i) (sngl(c_sw(i,j)),j=1,ny)
      end do

      close(14)
      close(15)
      close(16)
      close(17)
      close(18)
      close(19)
      close(25)
      close(26)
      close(27)
      close(28)
      close(29)
c      end if
203   format(1X,500(f14.8))
204   format(500(f10.4))
205   format(500(I4))

      return
      end


c #####################################################################
      subroutine friction_factor 
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'
      include 'pass.h'

c--- Local variables -------------------------------------------------
c      integer i,j
      real*8 a0, kn(nxm), lnfw, omega,d50

c=====================================================================

c--- Can do either a constant or varying friction factor
c--- Currently setup to do a constant friction factor

      d50=0.0002
      omega=2.d0*pi/period
      do j=1,ny
        do i=1,nx
c	   a0 = 0.5d0*period/pi*u0(i,j)
           kn(i)=2.5d0*d50
	   a0 = 0.5d0*g*H(i,j)/c_sw(i,j)/sigma
     .          /DCOSH(omega/c_sw(i,j)*ht(i,j))
	   if (a0.le.0.10 .or. ht(i,j).le.depthmin) a0=0.1
           lnfw = -5.977d0+5.213d0*(a0/kn(i))**(-0.194)
           fws = dexp(lnfw)
           shield=0.5d0*fws*(a0*omega)**2/(1.65)/g/d50
	   if (shield.le.0.05) then
              rough=2.5*d50
           else
           rough=170.d0*dsqrt(shield-0.05d0)*d50
           end if
           fcwij(i,j) = dexp(5.5d0*(rough/a0)**0.2d0-6.3d0)
           Pass_fw(i,j)=sngl(fcwij(i,j))
        end do
      end do


c      write(*,*)'fcw',(shield,i=1,nx)
    
c      do j=1,ny
c        do i=1,nx
cc	   a0 = 0.5d0*period/pi*u0(i,j)
c           kn(i)=2.5d0*.0002d0
c	   a0 = 0.5d0*g*H(i,j)/c_sw(i,j)/sigma
c     .          /DCOSH(omega/c_sw(i,j)*ht(i,j))
c	   if (a0.le.0.10 .or. ht(i,j).le.depthmin) a0=0.1
c           lnfw = -5.977d0+5.213d0*(a0/kn(i))**(-0.194)
c           fcwij(i,j) = dexp(lnfw)
c           Pass_fw(i,j)=sngl(fcwij(i,j))
cc           fcwij(i,j) = fcw
c        end do
c      end do

      open(29,file='fort.332',access='direct',recl=4*ny)

      do i=1,nx	 
	 write(29,rec=i) (sngl(fcwij(i,j)),j=1,ny)
      end do

      close(29)

      return
      end


c #####################################################################
c INIT OUTPUT
      subroutine init_output 
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'

c--- Local variables -------------------------------------------------
      integer i

c=====================================================================

c--- write file 'status.dat' basic values for MATLAB input
      open (unit=13,file='status.dat', status='unknown')
      write(13,903) dx, dy, dt, cr
      write(13,904) nx, ny, ntime, interval
      write(13,903) fcw, fcw, fcw, fcw
      
c!!! jump== no. of dt at which we print the time series

      write(6,*) 'Open output files for model results'
        do i=1,nsensor
          if (xsensor(i).ne.0 .and. ysensor(i).ne.0) then
            write(i+50,997) xsensor(i), ysensor(i), jump
            write(13,904) i+50, xsensor(i), ysensor(i), jump
          endif
        end do

      close(13)

903   format(4F12.6)
904   format(4I12)
996   format( 2x, 3(f15.10,2x))
997   format( 2x, 3(I15,2x))

      return
      end


c #####################################################################
c INITIAL TIME STEPS
      subroutine initialsteps                   
c #####################################################################
CDIR$ FLOW
        include 'pass.h'
        include 'winc_std_common.inc'

c--- Local functions -------------------------------------------------
      real*8 cel,ux,uy,momx1,momx2,momy1,momy2,
     . force_xx, force_xy, force_yx, force_yy

c--- Local variables -------------------------------------------------

c      integer i,j,ncode,nxflag

      real temp1(nxm,nym), temp2(nxm,nym), temp3(nxm,nym)

      real*8 h1, z1, t, qx1, qy1,
     . qxqy(nxm,nym), qxqx(nxm,nym), qyqy(nxm,nym),
     . qxqxdx(nxm,nym), qxqydx(nxm,nym), 
     . qyqydy(nxm,nym), qxqydy(nxm,nym), 
     . dqxdxn(nxm,nym), 
     . dqydyn(nxm,nym), 
     . dzetadxn(nxm,nym), dzetadyn(nxm,nym),
     . frictx(nxm,nym), fricty(nxm,nym),vs(nxm,nym),
     . fake(nxm,nym), fake1(nxm,nym), fake2(nxm,nym),
     . momxnm1(nxm,nym), momxnm2(nxm,nym), dtauxxdx(nxm,nym),
     . momynm1(nxm,nym), momynm2(nxm,nym),tauyy(nxm,nym),
     . dtauxydx(nxm,nym),dtauxydy(nxm,nym),tauxx(nxm,nym),
     . factor,tauxy(nxm,nym),dtauyydy(nxm,nym),
     . Uo(nxm,nym), Vo(nxm,nym), dUo_dx(nxm,nym), dUo_dy(nxm,nym),
     . dVo_dx(nxm,nym), dVo_dy(nxm,nym)
c=====================================================================
c ## define functions that will be used through out the routine -----
c
c ## WARNING: they look like variables but behave like functions,
c ## and must follow imediately the dimension statements
c----------------------------------------------------------------------
      cel(h1,z1) = DSQRT(g*(h1+z1))
      ux(qx1,z1,h1) = qx1/(h1+z1)
      uy(qy1,z1,h1) = qy1/(h1+z1)

        force_xx(i,j,t)= dSxxdxn(i,j)*(1-kold_start)
        force_xy(i,j,t)= dSxydyn(i,j)*(1-kold_start)
        force_yx(i,j,t)= dSxydxn(i,j)*(1-kold_start)
        force_yy(i,j,t)= dSyydyn(i,j)*(1-kold_start)

      momx1(i,j,t) =
     .   - force_xx(i,j,t) 
     .   - force_xy(i,j,t)
     .   - frictx(i,j)
     .   + dtauxydy(i,j)
     .   + dtauxxdx(i,j)

      momx2(i,j) = 
     .   -g*(ht(i,j)+zetan(i,j))*dzetadxn(i,j)
     .   - qxqxdx(i,j) -qxqydy(i,j)

      momy1(i,j,t) =
     .   - force_yx(i,j,t) 
     .   - force_yy(i,j,t)
     .   - fricty(i,j)
     .   + dtauxydx(i,j)
     .   + dtauyydy(i,j)

      momy2(i,j) =
     .   -g*(ht(i,j)+zetan(i,j))*dzetadyn(i,j)
     .   - qyqydy(i,j) -qxqydx(i,j)

c----------------------------------------------------------------------


      if(nx.gt.nxm) then
         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
         write(*,*)'nx is larger than nxm'
         write(*,*)'Change nxm in winc_std_common.inc and 
     .        winc_std_filter.f'
         stop
      end if
      if(ny.gt.nym) then
         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
         write(*,*)'ny is larger than nym'
         write(*,*)'Change nym in winc_std_common.inc and
     .        winc_std_filter.f'
         stop
      end if
      if(nym.gt.nmax.or.nxm.gt.nmax) then
         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
         write(*,*)'nmax should equal max(nxm,nym)'
         stop
      end if
c      if(5/delay.gt.(cnt-1)*cdt.and.tavgout.eq.1) then
c         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
c         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
c         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
c         write(*,*)'WARNING WARNING WARNING WARNING WARNING WARNING'
c         write(*,*)'Total run time less than 5*delay.'
c         write(*,*)'Time-averages will not be computed.'
c         write(*,*)'ntime*dt = ',ntime*dt
c         write(*,*)'5*delay = ',5/delay         
c      end if

c ## ----------------------------------------------------------------
c ## Calculating Quantities at t = -2*dt 
c ## ----------------------------------------------------------------
      write(*,902)' Initial steps'
      write(6,*)'t=-2dt'
      t = -2.d0*dt

      if (kold_start.eq.1) then
c-##---- Cold start
         do j = 1,ny
          do i = 1,nx
            zetan(i,j) = 0.d0
            qxn(i,j)   = 0.d0
            qyn(i,j)   = 0.d0
          end do
         end do

      elseif (kold_start.eq.0) then
c-##---- Hot start
           open(unit=19,file='hot2.dat',access='direct',
     .         status='old',recl=4*ny)
           do i=1,nx
             read(19,rec=i) (temp1(i,j),j=1,ny)
             read(19,rec=i+nx) (temp2(i,j),j=1,ny)
             read(19,rec=i+2*nx) (temp3(i,j),j=1,ny)
           end do
	   close(19)
	   do i=1,nx
	   do j=1,ny
	   	zetan(i,j)=dble(temp1(i,j))
	   	qxn(i,j)=dble(temp2(i,j))
	 	qyn(i,j)=dble(temp3(i,j))
	 end do
	 end do
       end if

c## Calculate the derivatives

      ncode =1
      nxflag = 1
      call deriv(zetan,qxn,qyn,dzetadxn,dzetadyn,dqxdxn,dqydyn,
     .           ncode,nxflag)

      do j=1,ny
        do i=1,nx
          qxqx(i,j) = qxn(i,j)*qxn(i,j)/(ht(i,j)+zetan(i,j))
          qxqy(i,j) = qxn(i,j)*qyn(i,j)/(ht(i,j)+zetan(i,j))
          qyqy(i,j) = qyn(i,j)*qyn(i,j)/(ht(i,j)+zetan(i,j))
          Uo(i,j) = ux(qxn(i,j),ht(i,j),zetan(i,j))
          Vo(i,j) = uy(qyn(i,j),ht(i,j),zetan(i,j))
        end do
      end do

      ncode =1
      nxflag = 1
      call deriv(qxqy,qxqx,qyqy,qxqydx,qxqydy,qxqxdx,qyqydy,
     .		 ncode,nxflag)

      call deriv(Uo, Vo, Vo, dUo_dx, dUo_dy, dVo_dx, dVo_dy,
     .           ncode,nxflag)
c ## eddy viscosity 
      if (kold_start.eq.1) then
      do j=1,ny
	 do i=1,nx
            dtauxydx(i,j) = 0.d0
            dtauxydy(i,j) = 0.d0
            dtauxxdx(i,j) = 0.d0
            dtauyydy(i,j) = 0.d0
	 end do	 
      end do
      else
      do j=1,ny
	 do i=1,nxa(j)-1
           vs(i,j)=Cs*Cs*dx*dy*sqrt(dUo_dx(i,j)*dUo_dx(i,j)*2.d0
     .           +dVo_dy(i,j)*dVo_dy(i,j)*2.d0
     .           +(dUo_dy(i,j)+dVo_dx(i,j))*(dUo_dy(i,j)+dVo_dx(i,j)))
	    tauxy(i,j) = (v_t(i,j)+vs(i,j))*(ht(i,j)+zetan(i,j))*
     .			 (dVo_dx(i,j)+dUo_dy(i,j))
            tauxx(i,j) = vs(i,j)*(ht(i,j)+zetan(i,j))*
     .                   (dUo_dx(i,j)*2.d0)
            tauyy(i,j) = vs(i,j)*(ht(i,j)+zetan(i,j))*
     .                   (dVo_dy(i,j)*2.d0)
	 end do
	 do i=nxa(j),nx
            tauxy(i,j) = 0.d0
            tauxx(i,j) = 0.d0
            tauyy(i,j) = 0.d0
            vs(i,j)=0.d0
	 end do	 
      end do
      call deriv(tauxy,tauxx,tauyy,dtauxydx,dtauxydy,dtauxxdx,
     .            dtauyydy,ncode,nxflag)
      end if

c ## call bottom friction

      factor=DTANH(t*delay)
      call bottom_friction(zetan,qxn,qyn,frictx,fricty,factor,0)

c ## calculate RHS of continuity & momentum eqs' ---------------------

      do j = 1,ny
        do i = 1,nx
                   
          cnm2(i,j) = - (dqxdxn(i,j) + dqydyn(i,j))
        
          momxnm2(i,j) = momx1(i,j,t)
          mxnm2(i,j) = momx1(i,j,t) + momx2(i,j)

          momynm2(i,j) = momy1(i,j,t)
          mynm2(i,j) = momy1(i,j,t) + momy2(i,j)

       end do
      end do


c ## Moving shoreline bound. condition
c !! set the forcing beyond any initially dry point to 'Zero'
         do j = 1,ny
            do i=nxa(j)+1,nx
              cnm2(i,j)=0.d0
              momxnm2(i,j)=0.d0
              mxnm2(i,j)=0.d0
              momynm2(i,j)=0.d0
              mynm2(i,j)=0.d0
              qxn(i,j)=0.d0
              qyn(i,j)=0.d0
              zetan(i,j)=-ht(i,j)+0.0001d0
	      dSxxdxn(i,j)=0.d0
	      dSxydyn(i,j)=0.d0
	      dSxydxn(i,j)=0.d0
	      dSyydyn(i,j)=0.d0
            end do
         end do



c## CHARACTERISTICS

      if ((ibc1.eq.3).or.(ibc1.eq.2)) then
        do j = 1,ny
          betan(1,j) = (ux(qxn(1,j),zetan(1,j),ht(1,j)) - 2.0d0*
     .              cel(zetan(1,j),ht(1,j)))
          betan(2,j) = (ux(qxn(2,j),zetan(2,j),ht(2,j)) - 2.0d0*
     .              cel(zetan(2,j),ht(2,j)))
          betan(3,j) = (ux(qxn(3,j),zetan(3,j),ht(3,j)) - 2.0d0*
     .              cel(zetan(3,j),ht(3,j)))
        end do

        call betachar(bnm2,betan,zetan,qxn,qyn,dqydyn,dzetadyn,
     .              momxnm2)
      end if



c ## ---------------------------------------------------------------           
c ## Calculating Quantities at t = -dt 
c ## ---------------------------------------------------------------  
      write(6,*)'t=-dt'
      t = -dt

c ## Initial conditions

      if (kold_start.eq.1) then
c-##---- Cold start
         do j = 1,ny
          do i = 1,nx
            zetan(i,j) = 0.d0
            qxn(i,j)   = 0.d0
            qyn(i,j)   = 0.d0
          end do
         end do
      elseif (kold_start.eq.0) then
c-##---- Hot start
          open(unit=19,file='hot1.dat',access='direct',
     .         status='old',recl=4*ny)
           do i=1,nx
             read(19,rec=i) (temp1(i,j),j=1,ny)
             read(19,rec=i+nx) (temp2(i,j),j=1,ny)
             read(19,rec=i+2*nx) (temp3(i,j),j=1,ny)
           end do
	   close(19)
	   do i=1,nx
	   do j=1,ny
	   	zetan(i,j)=dble(temp1(i,j))
	   	qxn(i,j)=dble(temp2(i,j))
	 	qyn(i,j)=dble(temp3(i,j))
	 end do
	 end do
      end if

c## Calculate the derivatives

      ncode =1
      nxflag = 1
      call deriv(zetan,qxn,qyn,dzetadxn,dzetadyn,dqxdxn,dqydyn,
     .           ncode,nxflag)

      do j=1,ny
        do i=1,nx
          qxqx(i,j) = qxn(i,j)*qxn(i,j)/(ht(i,j)+zetan(i,j))
          qxqy(i,j) = qxn(i,j)*qyn(i,j)/(ht(i,j)+zetan(i,j))
          qyqy(i,j) = qyn(i,j)*qyn(i,j)/(ht(i,j)+zetan(i,j))
          Uo(i,j) = ux(qxn(i,j),ht(i,j),zetan(i,j))
          Vo(i,j) = uy(qyn(i,j),ht(i,j),zetan(i,j))
        end do
      end do

      ncode =1
      nxflag=1
      call deriv(qxqy,qxqx,qyqy,qxqydx,qxqydy,qxqxdx,qyqydy,
     .		 ncode,nxflag)
      call deriv(Uo, Vo, Vo, dUo_dx, dUo_dy, dVo_dx, dVo_dy,
     .           ncode,nxflag)

c ## eddy viscosity 
      if (kold_start.eq.1) then
      do j=1,ny
	 do i=1,nx
            dtauxydx(i,j) = 0.d0
            dtauxydy(i,j) = 0.d0
            dtauxxdx(i,j) = 0.d0
            dtauyydy(i,j) = 0.d0
	 end do	 
      end do
      else

      do j=1,ny
	 do i=1,nxa(j)-1
            vs(i,j)=Cs*Cs*dx*dy*sqrt(dUo_dx(i,j)*dUo_dx(i,j)*2.d0
     .           +dVo_dy(i,j)*dVo_dy(i,j)*2.d0
     .           +(dUo_dy(i,j)+dVo_dx(i,j))*(dUo_dy(i,j)+dVo_dx(i,j)))
	    tauxy(i,j) = (v_t(i,j)+vs(i,j))*(ht(i,j)+zetan(i,j))*
     .			 (dVo_dx(i,j)+dUo_dy(i,j))
            tauxx(i,j) = vs(i,j)*(ht(i,j)+zetan(i,j))*
     .                   (dUo_dx(i,j)*2.d0)
            tauyy(i,j) = vs(i,j)*(ht(i,j)+zetan(i,j))*
     .                   (dVo_dy(i,j)*2.d0)
	 end do
	 do i=nxa(j),nx
            tauxy(i,j) = 0.d0
            tauxx(i,j) = 0.d0
            tauyy(i,j) = 0.d0
            vs(i,j)=0.d0
	 end do	 
      end do
      call deriv(tauxy,tauxx,tauyy,dtauxydx,dtauxydy,dtauxxdx,
     .            dtauyydy,ncode,nxflag)
      end if

c ## call bottom friction

      factor=DTANH(t*delay)
      call bottom_friction(zetan,qxn,qyn,frictx,fricty,factor,0)

c ## calculate RHS of continuity & momentum eqs'

      do j = 1,ny
        do i = 1,nx
                   
          cnm1(i,j) = - (dqxdxn(i,j) + dqydyn(i,j))
                  
          momxnm1(i,j) = momx1(i,j,t)
          mxnm1(i,j) = momx1(i,j,t) + momx2(i,j)

          momynm1(i,j) = momy1(i,j,t)
          mynm1(i,j) = momy1(i,j,t) + momy2(i,j)

        end do
      end do

c ## Moving shoreline bound. condition
      if (ibc2.eq.6) then
         do j = 1,ny
            do i=nxa(j)+1,nx
              cnm1(i,j)=0.d0
              momxnm1(i,j)=0.d0
              mxnm1(i,j)=0.d0
              momynm1(i,j)=0.d0
              mynm1(i,j)=0.d0
              qxn(i,j)=0.d0
              qyn(i,j)=0.d0
              zetan(i,j)=-ht(i,j)+0.0001d0
            end do
         end do
      end if

c## CHARACTERISTICS

      if ((ibc1.eq.2).or.(ibc1.eq.3)) then
        do j = 1,ny
          betan(1,j) = (ux(qxn(1,j),zetan(1,j),ht(1,j)) - 2.0d0*
     .              cel(zetan(1,j),ht(1,j)))
          betan(2,j) = (ux(qxn(2,j),zetan(2,j),ht(2,j)) - 2.0d0*
     .              cel(zetan(2,j),ht(2,j)))
          betan(3,j) = (ux(qxn(3,j),zetan(3,j),ht(3,j)) - 2.0d0*
     .              cel(zetan(3,j),ht(3,j)))
        end do
      end if


      call betachar(bnm1,betan,zetan,qxn,qyn,dqydyn,dzetadyn,
     .              momxnm1)



c ##------------------------------------------------------------------
c ## Calculating initial conditions at t=0 
c ##------------------------------------------------------------------
      write(6,*)'t=0'
      t = 0.d0

c ## Initial conditions

      if (kold_start.eq.1) then
c-##---- Cold start
         do j = 1,ny
          do i = 1,nx
            zetan(i,j) = 0.d0
            qxn(i,j)   = 0.d0
            qyn(i,j)   = 0.d0
          end do
         end do
      elseif (kold_start.eq.0) then
c-##---- Hot start
          open(unit=19,file='hot0.dat',access='direct',
     .         status='old',recl=4*ny)
           do i=1,nx
             read(19,rec=i) (temp1(i,j),j=1,ny)
             read(19,rec=i+nx) (temp2(i,j),j=1,ny)
             read(19,rec=i+2*nx) (temp3(i,j),j=1,ny)
           end do
	   close(19)
	   do i=1,nx
	   do j=1,ny
	   	zetan(i,j)=dble(temp1(i,j))
	   	qxn(i,j)=dble(temp2(i,j))
	 	qyn(i,j)=dble(temp3(i,j))
	 end do
	 end do
         if (wc.ne.0) then
          call shortwave_driver
          call wave_forcing
	end if
      end if
     
c ## Moving shoreline bound. condition
      if (ibc2.eq.6) then
         do j = 1,ny
            do i=nxa(j)+1,nx
              qxn(i,j)=0.d0
              qyn(i,j)=0.d0
              zetan(i,j)=-ht(i,j)+0.0001d0
            end do
         end do
      end if

c  Characteristix


      if ((ibc1.eq.2).or.(ibc1.eq.3)) then
        do j = 1,ny
          betan(1,j) = (ux(qxn(1,j),zetan(1,j),ht(1,j)) - 2.0d0*
     .              cel(zetan(1,j),ht(1,j)))
          betan(2,j) = (ux(qxn(2,j),zetan(2,j),ht(2,j)) - 2.0d0*
     .              cel(zetan(2,j),ht(2,j)))
          betan(3,j) = (ux(qxn(3,j),zetan(3,j),ht(3,j)) - 2.0d0*
     .              cel(zetan(3,j),ht(3,j))) 
        end do
      end if

902   format(/A)

      return
      end



c #####################################################################
      subroutine betachar(b_RHS,beta,zeta,qx,qy,dqydy,dzetady,
     .                    momx)
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'

c--- Local functions -------------------------------------------------
      real*8 cel

c--- Local variables -------------------------------------------------
      integer j
      real*8 b_RHS(nym), momx(nxm,nym), betatemp(nym), dbetady(nym),
     . beta(3,nym), zeta(nxm,nym), qx(nxm,nym), qy(nxm,nym), 
     . dqydy(nxm,nym), dzetady(nxm,nym),
     . h1, z1,dbetadx, c, inv_ht(nym), ux, uy

c=====================================================================
c---- statement functions---------------------------------------------
      cel(h1,z1) = DSQRT(g*(h1+z1))
c---------------------------------------------------------------------

      do j=1,ny
        betatemp(j)=beta(1,j)
      end do

      call spliney(ny,betatemp,dbetady)
      
      do j=1,ny

        dbetadx=(-3.d0*beta(1,j)+4.d0*beta(2,j)-beta(3,j))/(2.d0*dx)
        c = cel(zeta(1,j),ht(1,j))

        inv_ht(j) = 1.d0/(zeta(1,j)+ht(1,j))
        ux = qx(1,j)*inv_ht(j)
        uy = qy(1,j)*inv_ht(j)
        b_RHS(j)= -(ux-c)*dbetadx -uy*dbetady(j)
     .       +c*( (ht(1,j)+zeta(1,j))*dqydy(1,j) - qy(1,j)*
     .            dzetady(1,j) ) *inv_ht(j)**2.d0
     .       +momx(1,j)*inv_ht(j)
     .       +g*dhodxn(1,j)

      end do
      
      return
      end


c #####################################################################
      subroutine viscosity

c ## we're using a LINEARIZED VERSION (htot=ht) for the calculations
c    since that enables us to compute v_t only once at the
c    begining. However, it's not correct since the forcing is
c    "phased in"
c
c   With wave current feedback active everything gets updated so it is 
c   no longer linearized
c
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'
      include 'pass.h'

c--- Local variables -------------------------------------------------
      integer  index, iflag, ii, index1(10,nym)
      real*8 htot(nxm), 
     .       cg(nxm,nym),  gam, 
     .       newibr(nxm,nym)
c=====================================================================
c      write(6,*) 'Compute v_t'
c      write(6,903) 'vtshear :', vtshear
      Bo=0.125d0
c      Mdiss = 0.1d0

c	Initialize the breaking index
	do ii =1,4
	do j =1,ny
	  index1(ii,j)=0
	end do
	end do

c ---- input wave-group celerity from REFDIF results -----------------
c      open(unit=10, file='cg.dat')
cc!! Two aditional rows at the begining:
c      do i=1,2
c	read(10,203) (cg(i,j),j=1,ny)
c      end do
cc!! read the rest as before:
c      do i=1,nx
c	read(10,203) (cg(i,j),j=1,ny)
c      end do
c      close(10)

      do i=1,nx
         do j=1,ny
	    cg(i,j)=Pass_Cg(i,j)
         end do
      end do


      if(ibc3.eq.5.and.ibc4.eq.5) then
      do i=1,nx
         cg(i,1) = 0.25d0*cg(i,ny-1)+0.25d0*cg(i,ny)
     .             +0.25d0*cg(i,1)+0.25d0*cg(i,2)
         cg(i,ny) = cg(i,1)

      end do
      end if

      open(31,file='fort.331',access='direct',recl=4*ny)
      do i=1,nx
	write(31,rec=i) (sngl(cg(i,j)),j=1,ny)
      end do
99    format(200f10.4)
      close(31)

c ----- check for abnormal values of ibrk(i,j)	----------------------
c !!! This is not general! REFDIF is giving ibr=0 sometimes	
c !!! when it doesn't make sense!
      open(20,file='fort.317',access='direct',recl=4*ny)
      do i=1,nx
        do j=2,ny-3
          if (ibrk(i,j).ne.ibrk(i,j+1) .and. ibrk(i,j).ne.ibrk(i,j-1))
     .		ibrk(i,j)=ibrk(i,j-1)
	  if (ibrk(i,j).ne.ibrk(i,j+1) .and. 
     .		ibrk(i,j+2).ne.ibrk(i,j+3)) then
	    ibrk(i,j+1)=1
	    ibrk(i,j+2)=1
	  end if
        end do
        do j=1,ny
            if (ht(i,j).le.depthmin)  ibrk(i,j)=1
        end do
c!!!! Quick fix for problem of ibr(j=1).ne.ibr(j=ny)
       if (ibrk(i,1).ne.ibrk(i,ny)) ibrk(i,1)=ibrk(i,ny)
	write(20,rec=i) (real(ibrk(i,j)),j=1,ny)
      end do
      close(20)


c-- make a continuous approximation to the step function 'ibr'
      do j=1,ny
c------ find breaking point: find first ibrk(i,j)=1
        iflag=0
        ii=1
        do i=1,nx-1
	   if (ibrk(i,j).eq.0 .and. ibrk(i+1,j).eq.1) then
	     index1(ii,j)=i+1
             ii=ii+1
	   elseif (ibrk(i,j).eq.1 .and. ibrk(i+1,j).eq.0) then
	     index1(ii,j)=i
             ii=ii+1
	   end if
	end do
        if (ii.eq.2) index1(ii,j)=nx
        if (ii.eq.4) index1(ii,j)=nx
        if (ii.eq.6) index1(ii,j)=nx
        do i=1,index1(1,j)
           newibr(i,j)= 1.d0-DTANH(DBLE(index1(1,j)-i)/5.d0)
	end do
        do i=index1(1,j),index1(2,j)
           newibr(i,j)= 1.d0
	end do
        do i=index1(2,j)+1,index1(3,j)
           newibr(i,j)= 1.d0+0.5*DTANH(DBLE(index1(2,j)+1-i)/5.d0)
     .                  -0.5*DTANH(DBLE(index1(3,j)-i)/5.d0)
	end do
        do i=index1(3,j),index1(4,j)
           newibr(i,j)= 1.d0
	end do
      end do

      open(41,file='fort.329',access='direct',recl=4*ny)

      do i=1,nx
	 write(41,rec=i) (sngl(newibr(i,j)),j=1,ny)
      end do
      close(41)



c---- calculate wave energy dissipation from REFDIF's formulation ----
c     'gam' is the parameter in REFDIF for the stable wave energy
c     default:  gam=0.4
c---- Check if dissipation<0 !!!
      open(32,file='fort.328',access='direct',recl=4*ny)

c      do i=1,2
         call average_3ptx(nx,ny,dissipation)
c      end do
      do i=1,nx
	write(32,rec=i) (sngl(dissipation(i,j)),j=1,ny)
      end do
      close(32)


c --- look for 1st breakpoint along each j and compute v_t  ------
      index = 1
      do j=1,ny
c------ find breaking point: find first ibrk(i,j)=1
        iflag=0
        do i=1,nx
c           htot(i) = ht(i,j)+zetan(i,j)
           htot(i) = ht(i,j)
	   if (ibrk(i,j).eq.1 .and. iflag.eq.0) then
	     index=i
	     iflag=1
	   end if
	end do
c------ calculate eddy viscosity  ------------------------------
c ## the value of v_t outside the surf was multiplied by a "matchingc ## function", in order to match the value of v_t just after breaking

	v_t(index,j) = vtshear*htot(index)*DSQRT(g*htot(index))
        do i=1,nx
c ------- outside the surf-zone ---------
	  if (i.lt.index) then
	     v_t(i,j) = vtshear*DSQRT(0.5d0*fcwij(i,j))*htot(i)*u0(i,j)
	  else
c ------- inside the surf-zone ---------
	     if (ht(i,j).le.depthmin) then
	       if (ibrk(i,j).eq.0) then
	         v_t(i,j) = vtshear*DSQRT(0.5d0*fcwij(i,j))*depthmin
     .             *u0(i,j)
               else
	         v_t(i,j) = vtshear*DSQRT(0.5d0*fcwij(i,j))*depthmin
     .           *u0(i,j)+Mdiss*(dissipation(i,j))**(1.0/3.0)*depthmin
               end if
 
             else
	       if (ibrk(i,j).eq.0) then
	         v_t(i,j) = vtshear*DSQRT(0.5d0*fcwij(i,j))*htot(i)
     .             *u0(i,j)
               else
	         v_t(i,j) = vtshear*DSQRT(0.5d0*fcwij(i,j))*htot(i)
     .            *u0(i,j)+Mdiss*(dissipation(i,j))**(1.0/3.0)*htot(i)
               end if
	     end if
	  end if
        end do


c-- correction of v_t accounting for the smoothing transition 
c-- between breaking and non-breaking
        do i=1,index1(1,j)
           v_t(i,j)= v_t(i,j) + (v_t(index1(1,j),j)-v_t(i,j))*
     .              newibr(i,j)
	end do
        do i=index1(2,j)+1,index1(3,j)
           v_t(i,j)= v_t(i,j) + (v_t(index1(3,j),j)-v_t(i,j))*
     .              newibr(i,j)
	end do

      end do



c ## 3point average filter for v_t ----------------------------
c      do i=1,3
         call average_3ptx(nx,ny,v_t)
c      end do
      if(ibc3.eq.5.and.ibc4.eq.5) then
         call average_5pty(nx,ny,v_t)
      else
         call average_5ptyold(nx,ny,v_t)
      end if
c-- along the x-direction for each j


c ## cut-off upper limit on v_t from Svendsen's estimates
      do j=1,ny
        do i=1,nxa(j)
	   if (v_t(i,j).gt.0.04*ht(i,j)*DSQRT(g*ht(i,j))) then
	      v_t(i,j) = 0.04*ht(i,j)*DSQRT(g*ht(i,j))
           end if
ck 9/22/99     add ambient turbulence
           v_t(i,j)=v_t(i,j)+.0002d0

        end do
      end do

c      write(*,*)'v_t',(v_t(i,1),i=1,nx)

c ## write out some values -------------------------------------------
      open(16,file='fort.330',access='direct',recl=4*ny)
      do i=1,nx
	write(16,rec=i) (sngl(v_t(i,j)),j=1,ny)
c	write(*,*)'crap', (sngl(v_t(i,j)),j=1,ny)
      end do
      close(16)


903   format(1X,A,F12.6)
203   format(1X,500(f14.8))

      return
      end



c #####################################################################
      subroutine bottom_friction(zeta,xflux,yflux,frict_x,frict_y,
     .				 factor,istep)
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'

c--- Local variables -------------------------------------------------
      integer i, j, istep, kkk
      real*8 zeta(nxm,nym),xflux(nxm,nym),yflux(nxm,nym),aux3,aux4,
     . frict_x(nxm,nym),frict_y(nxm,nym), Vby, Vbx,uoVb,
     . factor, Vk, aux1, aux2, miu, costheta, costheta2, Vbuo, Vbuoc,
     . beta1(21), beta2(21), intbeta1(nxm,nym), intbeta2(nxm,nym), Vb

c=====================================================================

      aux1 = fcw/pi
      aux3=(2/pi)**2
      aux4=1/pi
           do j=1,ny
         do i=1,nxa(j)
	    Vby = (yflux(i,j)-qwy(i,j)*factor)/(ht(i,j)+zeta(i,j))
	    Vbx = (xflux(i,j)-qwx(i,j)*factor)/(ht(i,j)+zeta(i,j))
c -------- Non-linear friction, Svendsen & Putrevu (1990)
            
            if(Vby.ne.0.and.Vbx.ne.0) then
               miu = datan2(Vby,Vbx)-theta(i,j)
            else
               miu = -theta(i,j)
            end if
	    Vb = dsqrt(Vbx**2+Vby**2)
	    Vk = Vb *dcos(miu)
	    intbeta1(i,j)=0.d0
	    intbeta2(i,j)=0.d0
	    Vbuo = Vb*Vb/(u0(i,j)*u0(i,j))
	    Vbuoc = 2.d0*Vk/u0(i,j)

c	Numerical integral for calculating beta1 and beta2

c	    beta1(1) = DSQRT(1.d0+Vbuo+Vbuoc)
c	    beta2(1) = beta1(1)
c            do kkk=2,21
c               costheta = dcos(DBLE(kkk-1)*0.1d0*pi)
c               costheta2 = costheta*costheta
c	       Vbuoc = 2.d0*Vk/u0(i,j)*costheta
c	       beta1(kkk) = DSQRT(costheta2+Vbuo+Vbuoc)
c	       beta2(kkk) = beta1(kkk)*costheta
c	       intbeta1(i,j) = 0.025d0*(beta1(kkk)+beta1(kkk-1))
c     .			+intbeta1(i,j)
c	       intbeta2(i,j) = 0.025d0*(beta2(kkk)+beta2(kkk-1))
c     .			+intbeta2(i,j)
c            end do

c	Ib's approximation for beta1 and beta2

           uoVb = Vb/u0(i,j)
            intbeta1(i,j)= dsqrt(4*aux4*aux4+uoVb*uoVb)
            intbeta2(i,j)=0.5d0*dcos(miu) 
     .                  *DTANH(4.d0*aux4*uoVb)



            aux2 = 0.5d0*fcwij(i,j)*u0(i,j)
	    frict_y(i,j) = factor*aux2*( Vby*intbeta1(i,j)
     .        + intbeta2(i,j)*u0(i,j)*dsin(theta(i,j)))
	    frict_x(i,j) = factor*aux2*( Vbx*intbeta1(i,j)
     .        + intbeta2(i,j)*u0(i,j)*dcos(theta(i,j)))
         end do

         do i=nxa(j)+1,nx
	    frict_y(i,j) = 0.d0
	    frict_x(i,j) = 0.d0
         end do
      end do


      return
      end


c #####################################################################
c #####################################################################
      subroutine average_3ptx(npx,npy,afdata)
c
c     This subroutine performs a 3-point averaging filter along the
c     x-direction for every y=const row.
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'

c--- Local variables -------------------------------------------------
      integer i, j, npx, npy
      real*8 fdata(nxm,nym), afdata(nxm,nym)
c=====================================================================

      do j=1,npy
         do i=1,npx
            fdata(i,j)=afdata(i,j)
         end do
         do i=2, npx-1
            afdata(i,j) = 0.25d0*(fdata(i-1,j)+fdata(i+1,j))
     .                    +0.5d0*fdata(i,j)
         end do
         afdata(1,j)=fdata(1,j)
         afdata(npx,j)=fdata(npx,j)
      end do

      return
      end


c #####################################################################
      subroutine average_3pty(npx,npy,afdata)
c
c     This subroutine performs a 3-point averaging filter along the
c     y-direction for every x=const row.
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'

c--- Local variables -------------------------------------------------
      integer i, j, npx, npy
      real*8 fdata(nxm,nym), afdata(nxm,nym)
c=====================================================================

      do i=1,npx
         do j=1,npy
           fdata(i,j)=afdata(i,j)
         end do
         do j=2,npy-1
            afdata(i,j) = 0.25d0*(fdata(i,j-1)+fdata(i,j+1))
     .                    +0.5d0*fdata(i,j)
         end do
         afdata(i,1)=fdata(i,1)
         afdata(i,npy)=fdata(i,npy)
      end do

      return
      end
c #####################################################################
c #####################################################################

c #####################################################################
      subroutine average_5pty(npx,npy,afdata)
c
c     This subroutine performs a 5-point averaging filter along the
c     y-direction for every x=const row.
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'

c--- Local variables -------------------------------------------------
      integer i, j, npx, npy
      real*8 fdata(nxm,nym), afdata(nxm,nym)
c=====================================================================

      do i=1,npx
         do j=1,npy
           fdata(i,j)=afdata(i,j)
         end do
         do j=3,npy-2
            afdata(i,j) = 0.2d0*(fdata(i,j-1)+fdata(i,j+1))+
     .           0.2d0*(fdata(i,j-2)+fdata(i,j+2))
     .                    +0.2d0*fdata(i,j)
         end do
         afdata(i,1)=0.2d0*(fdata(i,npy-1)+fdata(i,2))+
     .           0.2d0*(fdata(i,npy-2)+fdata(i,3))
     .                    +0.2d0*fdata(i,1)
         afdata(i,npy)=0.2d0*(fdata(i,npy-1)+fdata(i,2))+
     .           0.2d0*(fdata(i,npy-2)+fdata(i,3))
     .                    +0.2d0*fdata(i,npy)
         afdata(i,2)=0.2d0*(fdata(i,1)+fdata(i,3))+
     .           0.2d0*(fdata(i,npy-1)+fdata(i,4))
     .                    +0.2d0*fdata(i,2)
         afdata(i,npy-1)=0.2d0*(fdata(i,npy-2)+fdata(i,npy))+
     .           0.2d0*(fdata(i,npy-3)+fdata(i,2))
     .                    +0.2d0*fdata(i,npy-1)
      end do

      return
      end
c #####################################################################
c #####################################################################
      subroutine average_5ptyold(npx,npy,afdata)
c
c     This subroutine performs a 5-point averaging filter along the
c     y-direction for every x=const row.
c #####################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'

c--- Local variables -------------------------------------------------
      integer i, j, npx, npy
      real*8 fdata(nxm,nym), afdata(nxm,nym)
c=====================================================================

      do i=1,npx
         do j=1,npy
           fdata(i,j)=afdata(i,j)
         end do
         do j=3,npy-2
            afdata(i,j) = 0.2d0*(fdata(i,j-1)+fdata(i,j+1))+
     .           0.2d0*(fdata(i,j-2)+fdata(i,j+2))
     .                    +0.2d0*fdata(i,j)
         end do
         afdata(i,1)=fdata(i,1)
         afdata(i,npy)=fdata(i,npy)
         afdata(i,2)=fdata(i,2)
         afdata(i,npy-1)=fdata(i,npy-1)
      end do

      return
      end
c #####################################################################
 



c #####################################################################
      subroutine spline(xin,yin,n,y2)
c
c    This subroutine will compute the cubic spline for an array.
c    
c #####################################################################
      include 'winc_std_common.inc'

      integer n,i,kk
      real*8 xin(nxm),yin(nxm),y2(nxm),p,qn,sig,un,u(nxm)

      y2(1)=0.d0
      u(1)=0.d0
      do i=2,n-1
         sig=(xin(i)-xin(i-1))/(xin(i+1)-xin(i-1))
         p=sig*y2(i-1)+2.d0
         y2(i)=(sig-1.d0)/p
         u(i)=(6.d0*((yin(i+1)-yin(i))/(xin(i+1)-xin(i))
     .        -(yin(i)-yin(i-1))
     .        /(xin(i)-xin(i-1)))/(xin(i+1)-xin(i-1))-sig*u(i-1))/p
      end do
      qn=0.d0
      un=0.d0
      y2(n)=(un-qn*u(n-1))/(qn*y2(n-1)+1.d0)
      
      do kk=n-1,1,-1
         y2(kk)=y2(kk)*y2(kk+1)+u(kk)
      end do
      return
      end

c #####################################################################
      SUBROUTINE SPLINT(xin,yin,y2,n,xout,yout,index,merge,j)
c
c    This subroutine will compute values from the cubic spline
c    
c #####################################################################
      include 'winc_std_common.inc'

      integer n,i,kk,khi,klo,index,j
      real*8 xin(nxm),yin(nxm),y2(nxm),xout(nxm),yout(nxm),aa,bb,hh

      do i=index,index+merge
      KLO=1
      KHI=N
1     IF (KHI-KLO.GT.1) THEN
        kk=(KHI+KLO)/2
        IF(xin(kk).GT.xout(i))THEN
          KHI=kk
        ELSE
          KLO=kk
        ENDIF
      GOTO 1
      ENDIF
      hh=xin(KHI)-xin(KLO)
      IF (hh.EQ.0.) PAUSE 'Bad xin input.'
      aa=(xin(KHI)-xout(i))/hh
      bb=(xout(i)-xin(KLO))/hh
      yout(i)=aa*yin(KLO)+bb*yin(KHI)+
     *      ((aa**3-aa)*y2(KLO)+(bb**3-bb)*y2(KHI))*(hh**2)/6.

      end do
      RETURN
      END

