c23456789012345678901234567890123456789012345678901234567890123456789012

c     ------------------------------------------------------------------
c
c     Nearshore Prediction Model 
c 
c     developed by H.T. Ozkan-Haller
c
c     ------------------------------------------------------------------

      subroutine WaveModule()
      implicit none

c     Include definitions 

      include 'definitions.inc' 
      
      include 'pass.h'
      
      include 'constants.inc' 
      
      write(*,*)'Master_Start=',Master_Start

      if(Master_Start.eq.1) then

c       Beginning to read input file inp.dat 
            
        call readin

c       Open up the required i/o files
            
        call opens
      
c       Setup initial condition

        ntime=0
        call adapt_circ_sedi     
        call incon

        write(*,*)'Initial condition setup.'

c       Initialize time and levels

        nlevel=1
        time=0.

c       Interpolate to output time series at xloc at t=0

        if(nloc.gt.0) then
         call timeout(1)
        endif
      
      else
      
        write(*,*)'nlevel=',nlevel
      
c       Execute time loop 

        write(*,*)'Got to time loop'

c       Start time loop

        ntot=Master_dt*N_interval_CallCirc/dt
        write(*,*) 'ntot in wave model=', ntot

        do ntime=1,ntot  
      
         call adapt_circ_sedi
	    
         time=time+dt  
         ntim=int(time/dt)
	      
         call nextime
       
c        Interpolate to output time series

         dtinv=1./dt
       
          write(*,*)'Time level    ',ntim,'  completed.'
          write(*,*) 'WAVEH=', waveh(30,3), waveh(nx+1,3)
       
          if(nint(mod(float(ntim),dtint*dtinv)).EQ.0.)then
 
            write(*,*)'Time level    ',ntim,'  completed.'
            write(229,*)'Time level    ',ntim,'  completed.'
	
            if(nloc.gt.0) then
             call timeout(1)
            endif
          endif

c      End time loop 
      
       enddo
      
       call adapt_wave
      
      endif
      
      return
      end

c       -------------------------------------------------------------------

	subroutine adapt_circ_sedi
	implicit none

        include 'definitions.inc'

	include 'pass.h'
	
	include 'constants.inc'
	
	do j=1,ny+1
	do i=1,nx+1
	
	eta(i,j)=Intp_eta_Wave(i,j) 

	u(i,j)=Intp_U_Wave(i,j)
	v(i,j)=Intp_V_Wave(i,j)
	
	h(i,j)=Depth_Wave(i,j)
	d(i,j)=Depth_Wave(i,j)+eta(i,j)

        enddo
        enddo
	
	write(*,*)'u in wave model at time', time
	write(*,*)u(21,65),u(22,65)
	

        return
	end 

c       -------------------------------------------------------------------
      
      	subroutine adapt_wave
        implicit none 
	
        include 'definitions.inc'

	include 'pass.h'
	
	include 'constants.inc'
	

! --- pass1

	do j=1,ny+1
	do i=1,nx+1
	Pass_Sxx(i,j) = Sxx(i,j)
	Pass_Sxy(i,j) = Sxy(i,j)
	Pass_Syy(i,j) = Syy(i,j)
	
	Pass_MassFluxU(i,j)=(en(i,j)*cs(i,j))/c(i,j)
	Pass_MassFluxV(i,j)=(en(i,j)*sn(i,j))/c(i,j)
	
	Pass_Diss(i,j)=epsb(i,j)
	
	Pass_Theta(i,j)=(asin(sn(i,j))*180./pi)+180.
	
	Pass_Height(i,j)=waveh(i,j)
	Pass_ubott(i,j)=g*waveh(i,j)*k(i,j)
     &  /(2.*sigma(i,j)*cosh(k(i,j)*d(i,j)))
	Pass_Cg(i,j)=cg(i,j)
	Pass_C(i,j)=c(i,j)
	Pass_wavenum(i,j)=k(i,j)
	
	if(epsb(i,j).gt.7.) then
	Pass_ibrk(i,j)=1.
	else
        Pass_ibrk(i,j)=0.
	endif	

	enddo
	enddo

        return
	end 


c     ------------------------------------------------------------------
c
c     Subroutine INCON
c
c     In this routine the depth grid and, if applicable, the longshore
c     current is input. Also, the initial conditions for u, v and eta 
c     are specified.
c
c     ------------------------------------------------------------------

      subroutine incon
      implicit none 
      
c     local variables related to reading input bathymetry
      integer xnnd, ynnd
      real h1,h2
      common/expand2d/xnnd,ynnd,h1,h2

c     switch to catch xi gridding problem
      integer shoreline_sw
      
      include 'definitions.inc' 
      
      include 'pass.h'
      
      include 'constants.inc' 

c     +++++++++++++++++++++++++++++++++++++++
c     + read input files and set parameters +
c     +++++++++++++++++++++++++++++++++++++++

      capb=0.78
      gamma=0.45
      

c     wave field angle (radians) and angular frequency from 
c     peak wave frequency (fbar)
      th0=th0*pi/180.
      omega=2.*pi*fbar

      do j=1,ny+1
       do i=1,nx+1
        sigma(i,j)=omega
       enddo
      enddo

c     echo inout variables on the screen 

      call scout

c    +++++++++++++++++++++++++
c    + set up the model mesh +
c    +++++++++++++++++++++++++

c     setup computational x-shore coordinate       
      
      do i=1,(nx+1)
       x(i)=float(i-1)*lx/float(nx)
       write(230,*) x(i)
      enddo

c     setup uniform longshore coordinate
      do j=1,(ny+1)
       y(j)=float(j-1)*ly/float(ny)
       write(220,*)y(j)
      enddo

      do j=1,ny+1
       zeta(j)=0.
       beta(j)=0.
      enddo

c     compute components of x-direction grid spacing

c     original
c     alphab=-log(0.0001)/(lx/2.)**2

c     crazed
c     xstar         = "middle" of domain
c     alphaz,alphab =  stretching functions
      do j=1,ny+1
c        xstar(j) = lx/2.
         xstar(j) = (zeta(j) + lx + beta(j))/2.
         alphaz(j)=-log(0.1)/(xstar(j))**2
         alphab(j)=alphaz(j)*xstar(j)**2/(lx-xstar(j))**2
      enddo

c     write out middle of domain
      open(215,file='xstar.mt',status='unknown')  
      do j=1,ny+1
	   write(215,*) xstar(j)
      enddo
      close(215)

c     compute derivatives of grid functions
      call ddy2fd(alphaz,alphazy)
      call ddy2fd(alphab,alphaby)

c     Specify physical x-shore coordinate 
c     linear spacing
      do j=1,(ny+1)
        do i=1,(nx+1)
	  xi(i,j) = (lx + beta(j) - zeta(j))/lx*x(i) + zeta(j)
        enddo
      enddo

c     check the shoreline position against the
c     specified shoreline in zeta

      shoreline_sw = 0
      do j=1,ny+1
         if (abs(xi(1,j)-zeta(j)).gt.1.0) then
	     print *, 'Cross-shore coordinate has moved shoreline'
	     print *, 'i, j, xi, zeta ',i,j,xi(1,j),zeta(j)
	     shoreline_sw = 1
	 endif
      enddo
      

cJAS \/ 15Oct02 Print out the inital bathymetry
      open(215,file='dep.mt',status='unknown')
      do j=1,ny+1
       do i=1,nx+1
        write(215,*) xi(i,j), y(j) , h(i,j)
       enddo
      enddo
      close(215)
      open(215,file='zz.mt',status='unknown')  
      do j=1,ny+1
	   write(215,*) zeta(j)
      enddo
      close(215)
        open(215,file='bb.mt',status='unknown')  
	do j=1,ny+1
	   write(215,*) beta(j)
	enddo
	close(215)
      if (shoreline_sw.eq.1)  stop
cJas /\


c     ++++++++++++++++++++++++++++++++
c     + deal with initial conditions +
c     ++++++++++++++++++++++++++++++++

c     Specify initial edge waves, if present


c     Specify total water depth

c      do j=1,ny+1
c       do i=1,nx+1
c        d(i,j)=h(i,j)
c       enddo
c      enddo

   
       do j=1,(ny+1)
        k((nx+1),j)=omega**2/(g*sqrt(tanh(omega**2*h((nx+1),j)/g)))
       enddo
	
c     call shoreline routine to initialize variables      
     
      call shore(1)
      
c      call refraction routine


       call refract2d

c     specify initial energy 

      do j=1,ny+1
       waveh(nx+1,j)=wh
       en(nx+1,j)=rho*g*wh**2/8.   
      enddo
 
      do i=nx,1,-1
      
         do j=1,ny+1
	    epsb(i+1,j)=(3.*sqrt(pi)/16.)*rho*g*fbar*capb**3
     &   *waveh(i+1,j)**7/(gamma**4*d(i,j)**5)
            b4(j)=en(i+1,j)*cg(i+1,j)*sn(i+1,j)
            by4(j)=0.
         enddo

         call ddy2fd(b4,by4)

         do j=1,ny+1
            if (abs(by4(j)).le.0.001) by4(j)=0.0
            by(i+1,j)=by4(j)
         enddo
      
         do j=1,(ny+1)
          en(i,j)=((en(i+1,j)*cg(i+1,j)*cs(i+1,j))
     &    +(by(i+1,j)+epsb(i+1,j))*(xi(i+1,j)-xi(i,j)))
     &    /(cg(i,j)*cs(i,j)) 	
	  waveh(i,j)=sqrt(8.*abs(en(i,j))/(rho*g))
         enddo
       
      enddo 
      
      do j=1,ny+1
       en(1,j)=0.
       waveh(1,j)=0.
      enddo
        
c       do j=1,ny+1
c        do i=1,nx+1
c         en(i,j)=0.
c         waveh(i,j)=sqrt(8.*abs(en(i,j))/(rho*g))
c         epsb(i,j)=0.
c        enddo
c       enddo
   
c     initialize radiation stress tensor if not already read in
    
       do j=1,ny+1
        do i=1,nx+1
         sxx(i,j)=en(i,j)*(nkh(i,j)*(cs(i,j)**2+1.)-0.5)
         sxy(i,j)=en(i,j)*nkh(i,j)*sn(i,j)*cs(i,j)
         syy(i,j)=en(i,j)*(nkh(i,j)*(sn(i,j)**2+1.)-0.5)
        enddo
       enddo 

c      initialize offshore value of wavenumber

       if(icase.eq.4) then 
        do j=1,(ny+1)
         kxb(j)=k(nx+1,j)*cs(nx+1,j)
         kyb(j)=k(nx+1,j)*sn(nx+1,j)
        enddo
       endif
  
      
c     call shoreline routine to initialize variables      

      call shore(1)
      
c     Output initial values
      
      do i=1,(nx+1)
       write(298,100)(xi(i,j),j=1,(ny+1))
       write(266,100)(waveh(i,j),j=1,(ny+1))
       write(267,100)(u(i,j),j=1,(ny+1))
       write(268,100)(v(i,j),j=1,(ny+1))
       write(269,100)(h(i,j),j=1,(ny+1))
       write(277,100)(kx(i,j),j=1,(ny+1))
       write(278,100)(ky(i,j),j=1,(ny+1))
       write(273,100)(sigma(i,j),j=1,(ny+1))
      enddo

100   format(e18.10)
      return
      end


c     ------------------------------------------------------------------
c
c     Subroutine REFRACT2D 
c
c     In this routine the dispersion relation is solved for the 
c     wavenumber at every spatial location. 
c
c     In this routine, the bathymetry is allowed to be variable in 2D 
c
c     This routine is largely untested. Use with caution!
c
c     ------------------------------------------------------------------

      subroutine refract2d
      implicit none 
      
      include 'definitions.inc' 
      
      real kold,knew,ok,f,fk,arg
      integer ii
      
      include 'constants.inc' 
      
c     solve for wavenumber for all spatial locations 

      ok=omega**2/g
      do j=1,ny+1
       do i=1,nx+1
        kold=ok*h(i,j)*(tanh((ok*h(i,j))**(0.75))**(-2./3.))
c       start Newton-Raphson iteration
        do ii=1,200
         f=ok*h(i,j)-kold*tanh(kold)
         fk=-kold/cosh(kold)**2-tanh(kold)
         knew=kold-f/fk
         if((abs(knew-kold)/knew).LT.1.e-06) go to 22
         kold=knew
        enddo
cJAS \/ 04Apr03 added condition on computation of wavenumber
c       write(*,*)'Convergence failed at IC',i,j,knew,kold
c22     k(i,j)=knew/h(i,j)
c       c(i,j)=-omega/k(i,j)
        if (abs(h(i,j)).gt.0.001) then
           write(*,*)'Convergence failed at IC',i,j,knew,kold
        endif
        if (i.eq.2.or.i.eq.3) print *, i,j,k(i,j),h(i,j)
22      if (abs(h(i,j)).lt.0.001) then
           k(i,j)=10.0
        else
           k(i,j)=knew/h(i,j)
        endif 
        c(i,j)=-omega/k(i,j)
cJas /\
       enddo
      enddo
      

c     define two components of wavenumber vector at offshore boundary


      do j=1,(ny+1)

cTOH \/ 14Apr03 change wavenumber at curvy offshore boundary
c      ky((nx+1),j)=k((nx+1),j)*sin(th0)
c      kx((nx+1),j)=k((nx+1),j)*cos(th0)

       ky((nx+1),j)=k((nx+1),j)*sin((th0+atan(betay(j))))
       kx((nx+1),j)=k((nx+1),j)*cos((th0+atan(betay(j))))

cToh /\
      enddo

      do i=nx,1,-1

c     compute y-derivatives

cJAS \/ 08Apr03 Replacing FFT ddy with FD ddy
c
c      call ddy2(b3,1,by1)
c
c      do j=1,ny+1
c       b1(2,j)=kx(i+1,j)
c       b1(1,j)=0.
c      enddo

       if (ddy_sw.eq.0) then

         write(*,*)'No longer set up to do FFT derivatives'
	 
       elseif (ddy_sw.eq.1) then

         do j=1,ny+1
            b4(j)=kx(i+1,j)
            by4(j)=0.
         enddo

         call ddy2fd(b4,by4)

         do j=1,ny+1
            if (abs(by4(j)).le.0.001) by4(j)=0.0
            by(i+1,j)=by4(j)
         enddo

       endif

       do j=1,ny+1
cJas    by(i+1,j)=by1(2,j)
	if(i.eq.nx) then
	 bx(i+1,j)=0.
	else
	 bx(i+1,j)=(kx(i+1,j)-kx(i+2,j))/(x(i+1)-x(i+2))
	endif
       enddo
       
c     find ky by stepping in x
      
       do j=1,(ny+1)
        cn(i,j)=phiy(i,j)/phix(i)
        ky(i,j)=ky(i+1,j)-(xi(i+1,j)-xi(i,j))
     &  *(by(i+1,j)+cn(i+1,j)*bx(i+1,j))      
       enddo
    
c     deduce angle of incidence from kx and ky values
      
       do j=1,(ny+1)
        arg=k(i,j)**2-ky(i,j)**2
cJAS \/
        if(arg.LT.0.) write(*,*)'arg=',i,j,arg,' will run into trouble!'
        kx(i,j)=sqrt(arg)

c       if(arg.GE.0.) then
c           kx(i,j)=sqrt(arg)
c       else
c         if (abs(arg).lt.0.001) then
c           kx(i,j)=0.0;
c         else
c           write(*,*)'arg=',i,j,arg,' will run into trouble!'
c         endif
c       endif
cJas /\
       enddo
              
c     finally ending i-loop
      enddo

      do j=1,(ny+1)
       do i=1,(nx+1)
        sn(i,j)=ky(i,j)/k(i,j)
        cs(i,j)=kx(i,j)/k(i,j)

        sigma(i,j)=omega
        nkh(i,j)=0.5*(1.+2.*k(i,j)*d(i,j)/sinh(2.*k(i,j)*d(i,j)))

        cg(i,j)=nkh(i,j)*c(i,j)
       enddo
      enddo

      do i=1,nx+1
       bigkx(i)=0.
       bigky(i)=0.
       do j=1,ny+1
        bigkx(i)=bigkx(i)+kx(i,j)
        bigky(i)=bigky(i)+ky(i,j)
       enddo
       bigkx(i)=bigkx(i)/(ny+1)
       bigky(i)=bigky(i)/(ny+1)
      enddo

      return
      end

c     ------------------------------------------------------------------
c
c     Subroutine NEXTIME  
c
c     Time stepping routine
c
c     ------------------------------------------------------------------

      subroutine nextime
      implicit none 
      
      real tmp1, tmp2, tmp3, tmp4, tmp5, tmp6, tmp7, tmp8, tmp9
      real tmp10, tmp11
      real tmp12, tmp13, sroot

      include 'definitions.inc' 
      include 'constants.inc' 

c     calling routine to carry out spatial differentiations

      call diff

c     update unknowns w1, w2, v, kx, ky, en
c     first time level
      
      tmp12 = dt/12.

      if(nlevel.EQ.1)then

       print *, 'You are using ',order_sw,' order BCs'

        do j=1,(ny+1)
         do i=1,(nx+1)
          en(i,j)=en(i,j)+dt*fen(i,j,1)
          if(en(i,j).LT.0.)en(i,j)=0.
         enddo
        enddo     
       if(icase.EQ.4)then
        do j=1,(ny+1)
         do i=1,(nx+1)
          kx(i,j)=kx(i,j)+dt*fkx(i,j,1)
          ky(i,j)=ky(i,j)+dt*fky(i,j,1)
         enddo
        enddo     
       endif

c     second time level

      elseif(nlevel.EQ.2)then

        do j=1,(ny+1)
         do i=1,(nx+1)
          en(i,j)=en(i,j)+dt*(1.5*fen(i,j,2)-0.5*fen(i,j,1))
          if(en(i,j).LT.0.)en(i,j)=0.
         enddo
        enddo

       if(icase.EQ.4)then
        do j=1,(ny+1)
         do i=1,(nx+1)
          kx(i,j)=kx(i,j)+dt*(1.5*fkx(i,j,2)-0.5*fkx(i,j,1))
          ky(i,j)=ky(i,j)+dt*(1.5*fky(i,j,2)-0.5*fky(i,j,1))
         enddo
        enddo
       endif

c     all other time levels

      elseif(nlevel.EQ.3)then

        do j=1,(ny+1)
         do i=1,(nx+1)
          en(i,j)=en(i,j)+(tmp12)*(23.*fen(i,j,3)-16.*fen(i,j,2)
     &    +5.*fen(i,j,1))
          if(en(i,j).LT.0.)en(i,j)=0.
         enddo
        enddo 

       if(icase.EQ.4)then
        do j=1,(ny+1)
         do i=1,(nx+1)
          kx(i,j)=kx(i,j)+(tmp12)*(23.*fkx(i,j,3)-16.*fkx(i,j,2)
     &    +5.*fkx(i,j,1))
          ky(i,j)=ky(i,j)+(tmp12)*(23.*fky(i,j,3)-16.*fky(i,j,2)
     &    +5.*fky(i,j,1))
         enddo
        enddo 
       endif

      endif

c     time stepping complete, roll back values 

      if(nlevel.EQ.3)call rollb
      if(nlevel.LT.3)nlevel=nlevel+1


c     Boundary conditions for energy equation and wavenumber equations
      
      if(icase.EQ.3.or.icase.EQ.4)then

c      Offshore boundary conditions

       do j=1,(ny+1)
        waveh((nx+1),j)=wh
c *tanh(0.0245*omega*ntim*dt)**2
        en((nx+1),j)=(rho*g*waveh((nx+1),j)**2/8.)

        waveh(1,j)=sqrt(8.*abs(en(1,j))/(rho*g))
       enddo

c     Lateral boundary condition

       if(iwall.eq.1) then
        do i=1,(nx)
         waveh(i,ny+1)=waveh(i,1)
         en(i,ny+1)=en(i,1)
        enddo

       elseif(iwall.eq.2) then

cJAS \/ 21 Oct 02 Wall lateral bc
        tmp12 = 4./3.
        tmp13 = 1./3.
	do i=1,(nx)

         if (order_sw.eq.1) then

c            waveh(i,ny+1)=waveh(i,ny)
c            waveh(i,1)=waveh(i,2)
c            en(i,ny+1)=en(i,ny)
c            en(i,1)=en(i,2)

            waveh(i,ny+1)=wh
	    en(i,ny+1)=(rho*g*wh**2/8.)
       


         elseif (order_sw.eq.4) then

            waveh(i,ny+1)=(48.*waveh(i,ny)-36.*waveh(i,ny-1)
     &               +16.*waveh(i,ny-2)-3.*waveh(i,ny-3))/25.
            waveh(i,1)   =(48.*waveh(i,2)-36.*waveh(i,3)
     &               +16.*waveh(i,4)-3.*waveh(i,5))/25.
            en(i,ny+1)=(48.*en(i,ny)-36.*en(i,ny-1)
     &               +16.*en(i,ny-2)-3.*en(i,ny-3))/25.
            en(i,1)   =(48.*en(i,2)-36.*en(i,3)
     &               +16.*en(i,4)-3.*en(i,5))/25.

         elseif (order_sw.eq.2) then

c           waveh(i,ny+1)=(tmp12)*waveh(i,ny)-(tmp13)*waveh(i,ny-1)
c           waveh(i,1)=(tmp12)*waveh(i,2)-(tmp13)*waveh(i,3)
           en(i,ny+1)=(tmp12)*en(i,ny)-(tmp13)*en(i,ny-1)
           en(i,1)=(tmp12)*en(i,2)-(tmp13)*en(i,3)

	   waveh(i,ny+1)=sqrt(8.*abs(en(i,ny+1))/(rho*g))
	   waveh(i,1)=sqrt(8.*abs(en(i,1))/(rho*g))

         endif

        enddo
       endif

       endif


      if(icase.EQ.4)then

c      Offshore boundary conditions
       do j=1,(ny+1)
        kx(nx+1,j)=kxb(j)
        ky(nx+1,j)=kyb(j)
       enddo
       

c     Lateral boundary condition

       if(iwall.eq.1) then
        do i=1,(nx)
         kx(i,ny+1)=kx(i,1)
         ky(i,ny+1)=ky(i,1)
        enddo
       elseif(iwall.eq.2) then
        tmp12 = 4./3.
        tmp13 = 1./3.
	do i=1,nx
         if (order_sw.eq.1) then
            kx(i,ny+1)=kx(i,ny)
            kx(i,1)   =kx(i,2)
            ky(i,ny+1)=ky(i,ny)
            ky(i,1)   =ky(i,2)
         elseif (order_sw.eq.4) then
            kx(i,ny+1)=(48.*kx(i,ny)-36.*kx(i,ny-1)
     &               +16.*kx(i,ny-2)-3.*kx(i,ny-3))/25.
            kx(i,1)   =(48.*kx(i,2)-36.*kx(i,3)
     &               +16.*kx(i,4)-3.*kx(i,5))/25.
            ky(i,ny+1)=(48.*ky(i,ny)-36.*ky(i,ny-1)
     &               +16.*ky(i,ny-2)-3.*ky(i,ny-3))/25.
            ky(i,1)   =(48.*ky(i,2)-36.*ky(i,3)
     &               +16.*ky(i,4)-3.*ky(i,5))/25.
         elseif (order_sw.eq.2) then
            kx(i,ny+1)=(tmp12)*kx(i,ny)-(tmp13)*kx(i,ny-1)
            kx(i,1)=(tmp12)*kx(i,2)-(tmp13)*kx(i,3)
            ky(i,ny+1)=(tmp12)*ky(i,ny)-(tmp13)*ky(i,ny-1)
            ky(i,1)=(tmp12)*ky(i,2)-(tmp13)*ky(i,3)
         endif
	enddo
       endif

      endif

c     Calling high-frequency noise filters

      call aliasx

      if (iwall.eq.1) then
          call aliasy
      elseif (iwall.eq.2) then
          call aliasy_w
      else
        print *, 'Not calling any aliasy routine'
      endif
cJas /\

c     recompute variables affected by filtering

       do j=1,(ny+1)
        do i=2,nx+1
         waveh(i,j)=sqrt(8.*abs(en(i,j))/(rho*g))
        enddo
       enddo
      
c     Update wavenumber and direction information 

      if(icase.EQ.4)then
       do j=1,(ny+1)
        do i=1,(nx+1)
         k(i,j)=sqrt(kx(i,j)**2+ky(i,j)**2)
         sn(i,j)=ky(i,j)/k(i,j)
         cs(i,j)=kx(i,j)/k(i,j)
        enddo
       enddo

       do j=1,(ny+1)
        do i=(nx+1),1,-1
         if(d(i,j).Gt.0.)then

          sigma(i,j)=sqrt(g*k(i,j)*tanh(k(i,j)*d(i,j)))
     &    -float(iwc)*(u(i,j)*kx(i,j)+v(i,j)*ky(i,j))

          gn(i,j)=2.*k(i,j)*d(i,j)/sinh(2.*k(i,j)*d(i,j))
         else
c          write(*,*)'Warning! Negative d of',d(i,j),' at',i,j
          sigma(i,j)=sigma(i+1,j)
          gn(i,j)=1.
         endif
      
         nkh(i,j)=0.5*(1.+gn(i,j))
         c(i,j)=-(sigma(i,j)+float(iwc)*(u(i,j)*kx(i,j)
     &   +v(i,j)*ky(i,j)))/k(i,j)
         cg(i,j)=nkh(i,j)*c(i,j)
        enddo
       enddo
      endif

c     Update radiation stress tensor      
      
      if(icase.EQ.3.or.icase.EQ.4)then
       do j=1,(ny+1) 
        do i=1,(nx+1)
         sxx(i,j)=en(i,j)*(nkh(i,j)*(cs(i,j)**2+1.)-0.5)
         sxy(i,j)=en(i,j)*nkh(i,j)*sn(i,j)*cs(i,j)
         syy(i,j)=en(i,j)*(nkh(i,j)*(sn(i,j)**2+1.)-0.5)
        enddo
       enddo
      endif
      
c     Outputting scnapshots
      
      if(ntim.ge.nfirst.and.mod(float(ntim-nfirst),
     &float(number)).EQ.0.)then
       do i=1,(nx+1)
        write(298,100)(xi(i,j),j=1,(ny+1))
        write(266,100)(waveh(i,j),j=1,(ny+1))
	write(267,100)(u(i,j),j=1,(ny+1))
	write(268,100)(v(i,j),j=1,(ny+1))
	write(269,100)(h(i,j),j=1,(ny+1))
        write(277,100)(kx(i,j),j=1,(ny+1))
        write(278,100)(ky(i,j),j=1,(ny+1))
        write(273,100)(sigma(i,j),j=1,(ny+1))
       enddo
      endif

c     Gate to prevent Nan's

      if(abs(eta(2,3)).GT.1.)then
       write(*,*)'Stopping at ntim=',ntim
       stop
      endif

100   format(e18.10)

      return
      end
c     ------------------------------------------------------------------
c
c     Subroutine DIFF  
c
c     In this routine spatial differentiations are carried out 
c     and the RHS's of the governing equations are computed
c
c     ------------------------------------------------------------------

      subroutine diff
      implicit none 
      
      include 'definitions.inc' 
      real sroot,tmp1,tmp2,helpen

      include 'constants.inc' 
      
c     calling x-derivative routine for several variables
         
       if(iwc.eq.1) then 
        do j=1,(ny+1)
         do i=1,nxp
          tmp1 = en(i,j)
          tmp2 = cg(i,j)
          en1(i,j) =tmp1*(u(i,j)+tmp2*cs(i,j))
          enb1(i,j)=tmp1*(v(i,j)+tmp2*sn(i,j))
         enddo
        enddo
       else 
        do j=1,(ny+1)
         do i=1,nxp
          tmp1 = en(i,j)*cg(i,j)
          en1(i,j) =tmp1*cs(i,j)
          enb1(i,j)=tmp1*sn(i,j)
         enddo
        enddo
       endif
       
       call ddxfd(en1,enx)
       call ddxfd(enb1,enbx)      
       call ddxfd(en,enax)
       

      if(icase.eq.4) then
       call ddxfd1(sigma,sigmax,x)
       call ddxfd1(kx,kxx,x)
       call ddxfd1(ky,kyx,x)
      endif
      
      if(iwc.eq.1) then
       call ddxfd(u,ux)
       call ddxfd(v,vx)
      endif

c     calling y-derivative routine
      
	 
       if(iwc.eq.1) then 
        do j=1,ny+1
         do i=1,(nx+1)
          en2(i,j)=en(i,j)*(v(i,j)+cg(i,j)*sn(i,j))
         enddo
        enddo
       else
        do j=1,ny+1
         do i=1,(nx+1)
          en2(i,j)=en(i,j)*cg(i,j)*sn(i,j)
         enddo
        enddo
       endif

         call ddyfd(en2,eny)

      if(icase.eq.4) then
          call ddyfd(sigma,sigmay)
      endif    
      
      if(iwc.eq.1) then
       call ddyfd(u,uy)
       call ddyfd(v,vy)
      endif
      
c     Specifying RHS's of governing equations for all points except those on shoreline
      
      do j=1,(ny+1)
       do i=2,(nx+1)
c        an(i,j)=phit(i,j)/phix(i)
c        bn(i,j)=phixi(i,j)/phix(i)
c        cn(i,j)=phiy(i,j)/phix(i)
	an(i,j)=0.
	bn(i,j)=1.
	cn(i,j)=0.
       enddo
      enddo

       do j=1,(ny+1)
        do i=2,(nx+1)
c     T&G model 1 for epsb       	
         epsb(i,j)=(3.*sqrt(pi)/16.)*rho*g*fbar*capb**3
     &   *waveh(i,j)**7/(gamma**4*d(i,j)**5)
c     W&T model for Superduck
c	 temp1=(3.*sqrt(pi)/16.)*rho*g*fbar*capb**3
c	 temp2=1+tanh(8.*(waveh(i,j)/(gamma*d(i,j))-0.99))
c	 temp3=1+(waveh(i,j)/(gamma*d(i,j)))**2
c         epsb(i,j)=temp1*(waveh(i,j)**3/d(i,j))*temp2
c     &	 *(1-(1/temp3**2.5))
	 
        enddo
       enddo

       do j=1,ny+1
        do i=2,nx+1
         helpen=float(iwc)*(sxx(i,j)*bn(i,j)*ux(i,j)+sxy(i,j)
     &   *(uy(i,j)+cn(i,j)*ux(i,j)+bn(i,j)*vx(i,j))+syy(i,j)
     &   *(vy(i,j)+cn(i,j)*vx(i,j)))

         fen(i,j,nlevel)=-an(i,j)*enax(i,j)-bn(i,j)*enx(i,j)
     &   -eny(i,j)-cn(i,j)*enbx(i,j)-helpen-epsb(i,j)

        enddo
       enddo

c       write(*,*) 'fen=',cn(64,3),fen(64,3,nlevel)

      if(icase.eq.4) then 
       do j=1,ny+1
        do i=2,nx+1
         fkx(i,j,nlevel)=-an(i,j)*kxx(i,j)+bn(i,j)*sigmax(i,j)
         fky(i,j,nlevel)=-an(i,j)*kyx(i,j)
     &   +(sigmay(i,j)+cn(i,j)*sigmax(i,j))
 
        enddo
       enddo
      endif
      
c     Shoreline boundary
      
      do j=1,(ny+1)
c       an(1,j)=phit(1,j)/phix(1)
c       bn(1,j)=phixi(1,j)/phix(1)
c       cn(1,j)=phiy(1,j)/phix(1)
	an(1,j)=0.
	bn(1,j)=1.
	cn(1,j)=0.
      enddo

      if(icase.EQ.3.or.icase.EQ.4)then
c       sroot = (3.*sqrt(pi)/16.)
        sroot = 0.33233509704478
        do j=1,ny+1
         epsb(1,j)=sroot*rho*g*fbar*capb**3*((waveh(1,j)
     &   /d(1,j))**5)*waveh(1,j)**2/(gamma**4)
        enddo
      endif

       do j=1,ny+1
        helpen=0.
 
c        helpen=float(iwc)*(sxx(1,j)*bn(1,j)*ux(1,j)+sxy(1,j)
c     &  *(uy(1,j)+cn(1,j)
c     &  *ux(1,j)+bn(1,j)*vx(1,j))+syy(1,j)*(vy(1,j)+cn(1,j)*vx(1,j)))
    
        fen(1,j,nlevel)=-an(1,j)*enax(1,j)-bn(1,j)*enx(1,j)-eny(1,j)
     &  -cn(1,j)*enbx(1,j)-helpen-epsb(1,j)

       enddo
     
      if(icase.eq.4) then 
        do j=1,ny+1
         fkx(1,j,nlevel)=-an(1,j)*kxx(1,j)+bn(1,j)*sigmax(1,j)
         fky(1,j,nlevel)=-an(1,j)*kyx(1,j)
     &   +(sigmay(1,j)+cn(1,j)*sigmax(1,j))     
        enddo
      endif


      return
      end

c     ------------------------------------------------------------------
c
c     Subroutine ROLLB  
c
c     In this routine the values of the RHS's of the equations are 
c     rolled back. This is necessary since only 3 time steps are kept 
c     in memory at any one time.
c
c     ------------------------------------------------------------------

      subroutine rollb
      implicit none 
      include 'definitions.inc' 
      include 'constants.inc' 

      do j=1,(ny+1)
       do i=1,(nx+1)

        fen(i,j,1)=fen(i,j,2)
        fkx(i,j,1)=fkx(i,j,2)
        fky(i,j,1)=fky(i,j,2)

        fen(i,j,2)=fen(i,j,3)
        fkx(i,j,2)=fkx(i,j,3)
        fky(i,j,2)=fky(i,j,3)

        fen(i,j,3)=0.
        fkx(i,j,3)=0.
        fky(i,j,3)=0.
       enddo
      enddo

      return
      end

c     ------------------------------------------------------------------
c
c     Subroutine ALIASY  
c
c     In this routine filtering of high frequency noise in the y-direction
c
c     ------------------------------------------------------------------

      subroutine aliasy
      implicit none 
      include 'definitions.inc'
      integer iorder,ist
      include 'constants.inc' 
      
      iorder=16
      ist=1

      do i=ist,(nx+1)
       do j=1,(ny+1)
        yw(j)=en(i,j)
        yyw(j)=0.
        ykx(j)=kx(i,j)-bigkx(i)
        yykx(j)=0.
        yky(j)=ky(i,j)-bigky(i)
        yyky(j)=0.
       enddo

       yw(ny+1)=yw(1)
       yyw(ny+1)=0.
       ykx(ny+1)=ykx(1)
       yykx(ny+1)=0.
       yky(ny+1)=yky(1)
       yyky(ny+1)=0.

       yw(ny+2)=yw(2)
       yyw(ny+2)=0.
       ykx(ny+2)=ykx(2)
       yykx(ny+2)=0.
       yky(ny+2)=yky(2)
       yyky(ny+2)=0.

 
c       call shapdw(yw,yyw,(ny+2),iorder,5)
c       call shapdw(ykx,yykx,(ny+2),iorder,5)
c       call shapdw(yky,yyky,(ny+2),iorder,5)


       do j=1,(ny+1)
c        en(i,j)=en(i,j)-yyw(j)
c        kx(i,j)=kx(i,j)-yykx(j)
c        ky(i,j)=ky(i,j)-yyky(j)
       enddo
      enddo

      return
      end
      
c     ------------------------------------------------------------------
c
c     Subroutine ALIASY_W
c
c     In this routine filtering of high frequency noise in the 
c     y-direction for wall lateral boundaries
c
c     ------------------------------------------------------------------
c
c     Checks for negative water depths (d).  If not too negative, 
c     ie, d > -0.0001 then reset d to zero and see what happens
c     else return back to calling routine.
c 
c     ------------------------------------------------------------------

      subroutine aliasy_w
      implicit none 
      real sroot
      include 'definitions.inc' 
      integer iorder,ist
      include 'constants.inc' 


      
      ist=1

      iorder=16
      do i=ist,(nx+1)
        do j=1,(ny+1)
          yw(j)=en(i,j)
          yyw(j)=0.
          ykx(j)=kx(i,j)-bigkx(i)
          yykx(j)=0.
          yky(j)=ky(i,j)-bigky(i)
          yyky(j)=0.
         enddo
      
c        call shapdw(yw,yyw,(ny+2),iorder,1)
c        call shapdw(ykx,yykx,(ny+2),iorder,1)
c        call shapdw(yky,yyky,(ny+2),iorder,1)

         do j=1,(ny+2)
c          en(i,j)=en(i,j)-yyw(i)
c          kx(i,j)=kx(i,j)-yykx(i)
c          ky(i,j)=ky(i,j)-yyky(i)
         enddo
      enddo

      return
      end


c     ------------------------------------------------------------------
c
c     Subroutine ALIASX  
c
c     In this routine filtering of high frequency noise in the x-direction
c
c     ------------------------------------------------------------------

      subroutine aliasx
      implicit none 
      real sroot
      integer iorder
      include 'definitions.inc' 
      include 'constants.inc' 
      
      iorder=16
      do j=1,(ny+1)
       do i=1,(nx+1)
        xd(i)=en(i,j)
        xxd(i)=0.
        xkx(i)=kx(i,j)-bigkx(i)
        xxkx(i)=0.
        xky(i)=ky(i,j)-bigky(i)
        xxky(i)=0.
       enddo
      
c       call shapdw(xd,xxd,(nx+1),iorder,1)
c       call shapdw(xkx,xxkx,(nx+1),iorder,1)
c       call shapdw(xky,xxky,(nx+1),iorder,1)

       do i=1,(nx+1)
c        en(i,j)=en(i,j)-xxd(i)
c        kx(i,j)=kx(i,j)-xxkx(i)
c        ky(i,j)=ky(i,j)-xxky(i)
       enddo
      enddo

      return
      end
c     ------------------------------------------------------------------
c
c     Subroutine SHAPDW
c
c     This routine applies a 1-d shapiro filter to the vector b1
c     and returns the correction field as bout
c
c     Input:
c
c     b1(ndim)            vector to be filtered
c     ndim                dimension of b1,bout   (ndim<200)
c     norder              order of the shapiro filter
c     scheme              flag for the type of boundary scheme to be used
c                         if scheme=
c                             1       no change at wall - constant order
c                             2       smoothing at wall - constant order
c                             3       no change at wall - reduced order
c                             4       smoothing at wall - reduced order
c                             5       periodic          - constant order
c                                     (1=ndm1 and 2=ndim)
c                         any other scheme is rejected
c
c     output:
c
c     bout(ndim)          correction field (not filtered field)
c
c     subroutines called : none
c
c     ------------------------------------------------------------------

      subroutine shapdw(b1,bout,ndim,norder,scheme)
      implicit none
      integer      ndim, norder, scheme
      REAL      b1(ndim),bn(1000),bout(ndim)

c  Local variables

      integer      i, i1, i2, k, kit, nord2, ndm1

      nord2 = norder/2
      ndm1 = ndim-1

c  scheme 1:  constant order and no change at wall

      if (scheme.eq.1) then
        do 120 k=1,nord2
          if(k.ne.nord2)  then
            bn(1) = 2.*(b1(1) - b1(2))
            bn(ndim) = 2.*(b1(ndim)-b1(ndm1))
          else
            bn(1) = 0.
            bn(ndim) = 0.
          end if
          do 100 i=2,ndm1
            bn(i) =(2*b1(i)-b1(i-1)-b1(i+1))
  100     continue
          do 110 i=1,ndim
            b1(i)=bn(i)
  110     continue
  120   continue
        do 130 i=1,ndim
          bout(i) = bn(i)*(0.5**norder)
  130   continue
        return

c  scheme 2:  constant order, smoothed at edges

      else if (scheme.eq.2) then
        do 160 k=1,nord2
          bn(1) = 2.*(b1(1) - b1(2))
          bn(ndim) = 2.*(b1(ndim)-b1(ndm1))
          do 140 i=2,ndm1
            bn(i) =(2*b1(i)-b1(i-1)-b1(i+1))
  140     continue
          do 150 i=1,ndim
            b1(i)=bn(i)
  150     continue
  160   continue
        do 170 i=1,ndim
          bout(i) = bn(i)*(0.5*norder)
  170   continue
        return

c  scheme 3:  reduced order and no change at wall

      else if (scheme.eq.3) then
        do 210 k=1,nord2
          kit = k
          i1 = k
          i2 = ndim-k+1
          if(k.eq.1) then
            do 180 i=2,ndm1
              bn(i) = 2*b1(i)-b1(i-1)-b1(i+1)
  180       continue
            bn(1) = 2.*(b1(1) - b1(2))
            bn(ndim) = 2.*(b1(ndim)-b1(ndm1))
          else
            do 190 i=i1,i2
              bn(i) = 2*b1(i)-b1(i-1)-b1(i+1)
  190       continue
          end if
          bout(i1) = bn(i1)*(0.5**(2.*kit))
          bout(i2) = bn(i2)*(0.5**(2.*kit))
          do 200 i=1,ndim
            b1(i) = bn(i)
  200     continue
  210   continue
        do 220 i=i1,i2
          bout(i) = bn(i)*(0.5**(2.*kit))
  220   continue
        bout(1) = 0.
        bout(ndim) = 0.
        return

c  scheme 4:  reduced order, smoothed at edges

      else if (scheme.eq.4) then
        do 260 k=1,nord2
          kit = k
          i1 = k
          i2 = ndim-k+1
          if(k.eq.1) then
            do 230 i=2,ndm1
              bn(i) = 2*b1(i)-b1(i-1)-b1(i+1)
  230       continue
            bn(1) = 2.*(b1(1) - b1(2))
            bn(ndim) = 2.*(b1(ndim)-b1(ndm1))
          else
            do 240 i=i1,i2
              bn(i) = 2*b1(i)-b1(i-1)-b1(i+1)
  240       continue
          end if
          bout(i1) = bn(i1)*(0.5**(2.*kit))
          bout(i2) = bn(i2)*(0.5**(2.*kit))
          do 250 i=1,ndim
            b1(i) = bn(i)
  250     continue
  260   continue
        do 270 i=i1,i2
          bout(i) = bn(i)*(0.5**(2.*kit))
  270   continue
        return

c  scheme 5:  constant order, periodic

      else if (scheme.eq.5) then
        do 300  k=1,nord2
          do 280  i=2,ndm1
            bn(i) = (2*b1(i)-b1(i-1)-b1(i+1))
  280     continue
          do 290  i=2,ndm1
            b1(i) = bn(i)
  290     continue
          b1(1) = b1(ndm1)
          b1(ndim) = b1(2)
  300   continue
        do 310  i=1,ndim
          bout(i) = b1(i)*(0.5**norder)
  310   continue
        return
      end if
      end


c     ------------------------------------------------------------------
c
c     Subroutine DDXFD1
c
c     Computes the  x-derivatives of uc using a second order
c     finite difference method.
c
c     ------------------------------------------------------------------

      subroutine ddxfd1(u,ud,x) 
      implicit integer(n)

      include 'parameter.inc'
      
      dimension u(nxp,nyp),ud(nxp,nyp),x(nxp)
      real u,ud,x
      
      do j=1,ny+1
       do i=1,nx
        ud(i,j)=(u(i+1,j)-u(i,j))/(x(i+1)-x(i))
       enddo
       ud(nx+1,j)=ud(nx,j)
      enddo
      
      return
      end
      
c     ------------------------------------------------------------------
c
c     Subroutine DDXFD
c
c     Computes the y-derivatives of u using a fourth order
c     finite difference method.
c
c     ------------------------------------------------------------------

      subroutine ddxfd(u,ud) 
      implicit integer(n)

      include 'parameter.inc'

c     need y(nyp) and iwall from definitions.inc
      common/trans/x(nxp),y(nyp),phi(nxp),phix(nxp),xi(nxp,nyp)
      common/switch/icase,iwall,iper,iwc,idis,ist,nc

c     declarations
   
      real x,y,phi,phix,xi
      integer icase,iwall,iper,iwc,idis,ist,nc
      real u(nxp,nyp),ud(nxp,nyp)
      real denom
      integer i,j

      denom=1.0/12.0

         do j=1,nyp

c           node at the wall
            ud(1,j) = denom*(-25.0*u(1,j) + 48.0*u(2,j) 
     &                     - 36.0*u(3,j) + 16.0*u(4,j) - 3.0*u(5,j))
     &                     /(x(2)-x(1))

c           node one in from the wall
            ud(2,j) = denom*(-10.0*u(2,j) - 3.0*u(1,j) +18.0*u(3,j)
     &                     - 6.0*u(4,j) + u(5,j))/(x(3)-x(2))

c           interior nodes
            do i=3,nxp-2
               ud(i,j) = denom*(u(i-2,j) - 8.0*u(i-1,j) 
     &                     + 8.0*u(i+1,j) - u(i+2,j))/(x(i+1)-x(i))
            enddo
     
c           node one in from the other wall
            ud(nx,j) = denom*( 10.0*u(nx,j) + 3.0*u(nx+1,j) 
     &        -18.0*u(nx-1,j) + 6.0*u(nx-2,j) - u(nx-3,j))
     &                     /(x(nx)-x(nx-1))

c           node at the other wall
            ud(nx+1,j) = denom*( 25.0*u(nx+1,j) - 48.0*u(nx,j) 
     &        + 36.0*u(nx-1,j) - 16.0*u(nx-2,j) + 3.0*u(nx-3,j))
     &                     /(x(nx+1)-x(nx))

         enddo

      return
      end

      
c     ------------------------------------------------------------------
c
c     Subroutine DDYFD
c
c     Computes the y-derivatives of u using a fourth order
c     finite difference method.
c
c     ------------------------------------------------------------------

      subroutine ddyfd(u,ud) 
      implicit integer(n)

      include 'parameter.inc'

      common/trans/x(nxp),y(nyp),phi(nxp),phix(nxp),xi(nxp,nyp)
      common/switch/icase,iwall,iper,iwc,idis,ist,nc

c     declarations
      real x,y,phi,phix,xi
      integer icase,iwall,iper,iwc,idis,ist,nc
      real u(nxp,nyp),ud(nxp,nyp)
      real uper(nxp,nyp+3)
      real denom,dely
      integer i,j

      denom=1.0/12.0

      if (iwall.eq.1) then
c        periodic boundary condition

c        fill in an array that is 2 rows longer
c        on both lateral ends, using periodicity of values
c        in the y-direction (1 -> ny)
c
c        uper(ny+4) = u(2)
c        uper(ny+3) = u(1)
c        uper(3:ny+2) = u(1:ny)
c        uper(2)     = u(ny) 
c        uper(1)     = u(ny-1)

         do i=1,nxp
            uper(i,1) = u(i,ny-1)
            uper(i,2) = u(i,ny)
            do j=3,ny+2
               uper(i,j) = u(i,j-2)
            enddo
            uper(i,ny+3) = u(i,1)
            uper(i,ny+4) = u(i,2)
         enddo

c        compute the derivative with a fourth order FD eqn

         do i=1,nxp
            do j=3,ny+2

               dely = y(j-1)-y(j-2)
               
               ud(i,j-2) = denom*(uper(i,j-2) - 8.0*uper(i,j-1) 
     &             + 8.0*uper(i,j+1) - uper(i,j+2))/dely

            enddo
 
cJAS \/ 23Apr03
c           HERE HERE HERE
c           add line here ud(ny+1)=ud(1)
c           SAME CHANGE IN DDYFD

c            ud(i,ny+1) = ud(i,1)
cJas /\
         enddo

      elseif (iwall.eq.2) then
c        wall boundary condition

         do i=1,nxp

c           node at the wall
            ud(i,1) = denom*(-25.0*u(i,1) + 48.0*u(i,2) 
     &                     - 36.0*u(i,3) + 16.0*u(i,4) - 3.0*u(i,5))
     &                     /(y(2)-y(1))

c           node one in from the wall
            ud(i,2) = denom*(-10.0*u(i,2) - 3.0*u(i,1) +18.0*u(i,3)
     &                     - 6.0*u(i,4) + u(i,5))/(y(3)-y(2))

c           interior nodes
            do j=3,ny-1
               ud(i,j) = denom*(u(i,j-2) - 8.0*u(i,j-1) 
     &                     + 8.0*u(i,j+1) - u(i,j+2))/(y(j+1)-y(j))
            enddo

c        node one in from the other wall
         ud(i,ny) = denom*( 10.0*u(i,ny) + 3.0*u(i,ny+1) 
     &     -18.0*u(i,ny-1) + 6.0*u(i,ny-2) - u(i,ny-3))
     &                  /(y(ny)-y(ny-1))

c        node at the other wall
         ud(i,ny+1) = denom*( 25.0*u(i,ny+1) - 48.0*u(i,ny) 
     &     + 36.0*u(i,ny-1) - 16.0*u(i,ny-2) + 3.0*u(i,ny-3))
     &                  /(y(ny+1)-y(ny))
     
         enddo
      else
 
         print *, "Unexpected iwall = ",iwall," in ddyfd."
         stop

      endif

      return
      end

c     ------------------------------------------------------------------
c
c     Subroutine DDY2FD
c
c     Computes the y-derivatives of a vector u using a fourth order
c     finite difference method.
c
c     ------------------------------------------------------------------

      subroutine ddy2fd(u,ud) 
      implicit integer(n)

      include 'parameter.inc'

      common/trans/x(nxp),y(nyp),phi(nxp),phix(nxp),xi(nxp,nyp)
      common/switch/icase,iwall,iper,iwc,idis,ist,nc

c     declarations
      real x,y,phi,phix,xi
      integer icase,iwall,iper,iwc,idis,ist,nc
      real u(nyp),ud(nyp)
      real uper(nyp+3)
      real denom,dely
      integer i,j

      denom=1.0/12.0

      if (iwall.eq.1) then
c        periodic boundary condition

c        fill in an array that is 2 rows longer
c        on both lateral ends, using periodicity of values
c        in the y-direction (1 -> ny)
c
c        uper(ny+4) = u(2)
c        uper(ny+3) = u(1)
c        uper(3:ny+2) = u(1:ny)
c        uper(2)     = u(ny) 
c        uper(1)     = u(ny-1)

         uper(1) = u(ny-1)
         uper(2) = u(ny)
         do j=3,ny+2
            uper(j) = u(j-2)
         enddo
         uper(ny+3) = u(1)
         uper(ny+4) = u(2)

c        compute the derivative with a fourth order FD eqn

         do j=3,ny+2

            dely = y(j-1)-y(j-2)
               
            ud(j-2) = denom*(uper(j-2) - 8.0*uper(j-1) 
     &          + 8.0*uper(j+1) - uper(j+2))/dely

         enddo

cJAS \/ 23Apr03
c        HERE HERE HERE
c        add line here ud(ny+1)=ud(1)
c        SAME CHANGE IN DDYFD
 
c         ud(ny+1) = ud(1)
cJas /\

      elseif (iwall.eq.2) then
c        wall boundary condition

c        node at the wall
         ud(1) = denom*(-25.0*u(1) + 48.0*u(2) 
     &                  - 36.0*u(3) + 16.0*u(4) - 3.0*u(5))
     &                  /(y(2)-y(1))

c        node one in from the wall
         ud(2) = denom*(-10.0*u(2) - 3.0*u(1) +18.0*u(3)
     &                  - 6.0*u(4) + u(5))/(y(3)-y(2))

c        interior nodes
         do j=3,ny-1
            ud(j) = denom*(u(j-2) - 8.0*u(j-1) 
     &                  + 8.0*u(j+1) - u(j+2))/(y(j+1)-y(j))
         enddo

c        node one in from the other wall
         ud(ny) = denom*( 10.0*u(ny) + 3.0*u(ny+1) 
     &     -18.0*u(ny-1) + 6.0*u(ny-2) - u(ny-3))
     &                  /(y(ny)-y(ny-1))

c        node at the other wall
         ud(ny+1) = denom*( 25.0*u(ny+1) - 48.0*u(ny) 
     &     + 36.0*u(ny-1) - 16.0*u(ny-2) + 3.0*u(ny-3))
     &                  /(y(ny+1)-y(ny))
      else
 
         print *, "Unexpected iwall = ",iwall," in ddyfd."
         stop

      endif

      return
      end

      
c     ------------------------------------------------------------------
c
c     Subroutine READIN
c
c     Reads input file inp.dat
c
c     ------------------------------------------------------------------
      
      subroutine readin
      implicit none

      character*132 tmp
      integer ipo
 
      include 'definitions.inc' 
      include 'constants.inc' 

      open(214,file='inp.dat')

c     Beginning to read input file inp.dat 

c     read timestep, total number of timesteps, number of snapshots
c          and first time snapshot is written
      read(214,'(a)') tmp
      read(214,*)dt,number,nfirst


c     read time interval for (nloc) snapshots and x,y positions 
c     for those snaps
      read(214,'(a)') tmp
      read(214,*)nloc, dtint

      if(nloc.gt.0) then
       read(214,'(a)') tmp
       do 40 ipo=1,nloc
        read(214,*) xpos(ipo),ypos(ipo)

 40    continue
      endif

      
c     icase = 1 - no shortwave forcing
c             2 - steady forcing
c             3 - integration of energy eqn
c             4 - integration of energy and wave eqn
c     idep  = 1 - sloping bottom
c             2 - bathymetry file input
c             3 - barred beach
c     ibc1  = 1 - moving shoreline 
c             2 - wall   
c     ibc2  = 1 - open boundary condition
c             2 - wall
c     iwall = 1 - periodic boundary conditions
c             2 - wall boundaries
c     iper  = 0 - no velocity perturbations
c             1 -    velocity perturbations
c     iwc   = 0 - no wave-current interation
c             1 -    wave-current interation
c     idis  = 1 - grid points at shoreline
c             2 - grid points at shoreline and open bndry
c             3 - uniform cross-shore grid points
      read(214,'(a)') tmp
      read(214,*)icase,iwall,iwc


c     lx        - cross-shore domain length
c     slopm0    - beach slope.  
c                 plane beach -> input slope;  flat bottom -> input 0
c                 variable depth -> choose idep wisely 
c     h0        - depth at shoreline.
c                 moving shoreline -> input 0
c     nc        - number of longshore wavelengths in domain
c     wkedge    - longshore wavenumber; ly = nc*(2pi/wkedge)
c     eps       - shoreline amplitude of initial edge wave
c     ist   = 0 - initial standing edge wave 
c             1 - initial progressive edge wave 
c     epsi      - threshold shoreline depth
      read(214,'(a)') tmp
      read(214,*)lx,ly



c     wh        - incident waves wave height
c     fbar      - incident waves peak frequency
c     th0       - incident waves angle of incidence
c     snell     - incident waves value of sin(th0/c0);  0 if icase != 2
      read(214,'(a)') tmp
      read(214,*)wh,fbar,th0



      if ((fbar.eq.0.0).and.(icase.ge.3)) then
         print *, 'You have chosen to integrate wave forcing'
         print *, 'icase = ',icase,' but input a 0 peak frequency'
         print *, 'fbar = ',fbar
         stop
      endif

c     ddw_sw = 0 - FFT d/dy computation (ddy.f)
c            = 1 - FD  d/dy computation (ddyfd.f)
      ddy_sw=1


c     order_sw  = 1 for 1st order boundary conditions in nextime.f
c               = 2 for 2nd order bcs
c               = 4 for 4th order bcs
      order_sw=2
      
      close(214)

      open(215,file='inp.mt',status='unknown')
      write(215,*) dt
      write(215,*) number
      write(215,*) nfirst
      write(215,*) nloc
      write(215,*) dtint
      if (nloc.gt.0) then
         do 10 ipo=1,nloc
            write(215,*) xpos(ipo)
 10      continue
         do 20 ipo=1,nloc
            write(215,*) ypos(ipo)
 20      continue
      endif

      write(215,*) icase
      write(215,*) iwall
      write(215,*) iwc
      write(215,*) lx
      write(215,*) ly
      write(215,*) wh
      write(215,*) fbar
      write(215,*) th0
      write(215,*) nx
      write(215,*) ny

      close(215)

      return
      end
    
c     ------------------------------------------------------------------
c
c     Subroutine SCOUT  
c
c     In this routine the depth grid and, if applicable, the longshore
c     current is input. Also, the initial conditions for u, v and eta 
c     are specified.
c
c     ------------------------------------------------------------------

      subroutine scout
      implicit none 
      
      real ramptime, rampnum

      include 'definitions.inc' 
      include 'constants.inc' 

      open(213,file='ok.log',status='unknown')

      write(213,*)'OKMODEL --- by H.T. Ozkan-Haller and J.T. Kirby'
      write(213,*)'  '
      write(213,*)'Reading input file'
      write(213,*)'Time step= ',dt,' and total number of time ',
     &'steps= ',ntot
      write(213,*)'Total simultation time = ', dt*ntot/60.0 ,' (min)'

      write(213,*)'Time series output will be written every ',dtint,' sec 
     &at the cross-shore location ',(xpos(i),i=1,nloc),' m.'

      write(213,*)'Snapshots of variables are output every ',number,' time
     &steps.'
      write(213,*)'The first snapshot output occurs at time step ',nfirst
      write(213,*)'  '
      write(213,*)'VALUES FOR SWITCHES'
      write(213,*)'  '
      write(213,*)'icase= ',icase

      if(icase.EQ.1.or.icase.eq.2)then
       write(213,*)'These options are no longer supported. Stopping!'
       stop
      elseif(icase.EQ.3.or.icase.EQ.4)then
       write(213,*)'Steady or unstaeady forcing, integration of the energy 
     & equation.'
      elseif(icase.eq.4) then
       write(213,*)'Unsteady forcing, integration of energy eqn and 
     & conservation of wavenumber' 
      else
       write(213,*)'Invalid value for icase specified. Stopping!'
       stop
      endif

      write(213,*)'  '

      if(iwall.EQ.1)then
       write(213,*)'Using periodic lateral boundaries.'
      elseif(iwall.EQ.2)then
       write(213,*)'Using wall lateral boundaries.'
      else
       write(213,*)'Invalid value for iwall specified. Stopping!'
       stop
      endif

      write(213,*)'  '

      if(iwc.EQ.0)then
       write(213,*)'Wave-current interaction excluded.'
      elseif(iwc.EQ.1)then
       write(213,*)'Wave-current interaction included.'
      else
       write(213,*)'Invalid value for iwc specified. Stopping!'
       stop
      endif

      write(213,*)'  '

      write(213,*)'DOMAIN PROPERTIES'
      write(213,*)'  '
      write(213,*)'Domain length in x= ',lx,' m.'
      write(213,*)'Domain width in y= ',ly,' m.'
      write(213,*)'  '
      write(213,*)'INCOMING SHORT WAVE PROPERTIES'
      write(213,*)'  '
      write(213,*)'Offshore wave height of short waves is ',wh,' m.'
      write(213,*)'Peak frequency  of short waves is ',fbar,' sec.'
      write(213,*)'Angle of incidence of short waves is ',th0*180./pi,' de
     &grees.'

      close(213)

      return
      end
      
c     ------------------------------------------------------------------
c
c     Subroutine TIMEOUT
c
c     Outputs time series at specified points 
c     and/or at specified longshore transects
c
c     ------------------------------------------------------------------
   
      subroutine timeout(narg)
      implicit none 
      
      include 'definitions.inc'
      
      integer ipo,ii,istep
	    
      include 'constants.inc'
     
      
c     Outoutting time serious at discrete points      
      
      if(narg.eq.1) then
      
       do ipo=1,nloc
        xloc(ipo)=xpos(ipo)
c     --finding y point that is nearest to specified ypos
        nypos(ipo)=int(ypos(ipo)*float(ny+1)/ly+0.5)
	if(nypos(ipo).eq.ny+1) nypos(ipo)=1
        nn1=nypos(ipo)
        yloc(ipo)=y(nn1)
      
        do i=1,(nx+1)
         if(xi(i,nn1).LE.xloc(ipo).AND.xi(i+1,nn1).GT.xloc(ipo)) 
     &   iloc(nn1)=i
        enddo
     
        if(icase.eq.3.or.icase.eq.4) then
	 whloc1(ipo)=waveh(iloc(nn1),nn1)+(waveh(iloc(nn1)+1,nn1)
     &   -waveh(iloc(nn1),nn1))*(xloc(ipo)-xi(iloc(nn1),nn1))
     &   /(xi(iloc(nn1)+1,nn1)-xi(iloc(nn1),nn1))
        endif

        if(icase.eq.4) then 
         sigmaloc1(ipo)=sigma(iloc(nn1),nn1)+(sigma(iloc(nn1)+1,nn1)
     &   -sigma(iloc(nn1),nn1))*(xloc(ipo)-xi(iloc(nn1),nn1))
     &   /(xi(iloc(nn1)+1,nn1)-xi(iloc(nn1),nn1))
         xkxloc1(ipo)=kx(iloc(nn1),nn1)+(kx(iloc(nn1)+1,nn1)
     &   -kx(iloc(nn1),nn1))*(xloc(ipo)-xi(iloc(nn1),nn1))
     &   /(xi(iloc(nn1)+1,nn1)-xi(iloc(nn1),nn1))
         xkyloc1(ipo)=ky(iloc(nn1),nn1)+(ky(iloc(nn1)+1,nn1)
     &   -ky(iloc(nn1),nn1))*(xloc(ipo)-xi(iloc(nn1),nn1))
     &   /(xi(iloc(nn1)+1,nn1)-xi(iloc(nn1),nn1))
        endif


       enddo
      
       if(ntim.eq.0) then
        if(icase.eq.3.or.icase.eq.4) then
            write(254,100)(xloc(ipo),ipo=1,nloc)
            write(254,100)(yloc(ipo),ipo=1,nloc)
        endif
        if(icase.eq.4) then
           do ii=6,8
              write(250+(ii-1),100)(xloc(ipo),ipo=1,nloc)
              write(250+(ii-1),100)(yloc(ipo),ipo=1,nloc)
           enddo
        endif
       endif
       
       if(icase.eq.3.or.icase.eq.4) then
        write(254,100)(whloc1(ipo),ipo=1,nloc)
       endif
       
       if(icase.eq.4) then
        write(255,100)(xkxloc1(ipo),ipo=1,nloc)
        write(256,100)(xkyloc1(ipo),ipo=1,nloc)
        write(257,100)(sigmaloc1(ipo),ipo=1,nloc)
       endif
         
      endif
      
100   format(20e16.8)
101   format(e16.8)

      return 
      end

c     ------------------------------------------------------------------
c
c     Subroutine OPENS 
c
c     Opens files for i/o depending on run inputs
c
c     ------------------------------------------------------------------

      subroutine opens
      implicit none 
      include 'definitions.inc'

c     +++++++++
c     + input +
c     +++++++++

c         open(227,file='zz.dat')
c         open(228,file='bb.dat')

      
c     ++++++++++
c     + output +
c     ++++++++++

c     always open for output
      open(220,file='y.dat')
      open(230,file='phi.dat')
      
      open(298,file='zeta.dat')
           
      open(269,file='hh.dat')
      open(266,file='waveh.dat')
      open(267,file='u.dat')
      open(268,file='v.dat')
      open(277,file='kx.dat')
      open(278,file='ky.dat')
      open(273,file='sig.dat')

      open(229,file='ok.status')
      
c     output values at a point in the domain
      if (nloc.gt.0) then

c        when integrating the energy or wave eqns
         if ((icase.eq.3).or.(icase.eq.4)) open(254,file='wht.dat')
 
c        when integrating the wave eqn
         if (icase.eq.4) then
            open(255,file='kxt.dat')
            open(256,file='kyt.dat')
            open(257,file='sigt.dat')
         endif
      
      endif

      
      return
      end

c     ------------------------------------------------------------------
c
c     Subroutine SHORE  
c
c     In this routine the position of the shoreline is computed given 
c     the velocities at the shoreline. 
c
c     ------------------------------------------------------------------

      subroutine shore
      implicit none 
      
      real tmp,tmp1,tmp2
      include 'definitions.inc' 
      include 'constants.inc' 

c     linear grid
      do j=1,(ny+1)
       do i=1,(nx+1)
 
	tmp = 1./(lx + beta(j) - zeta(j))
        phixi(i,j)=lx*tmp
        phiy(i,j)= (zetay(j) * (x(i)-lx) - betay(j)*x(i))*tmp
        phit(i,j)= 0.
        phix(i)=1.
       enddo
      enddo 
      

      return
      end
