c23456789012345678901234567890123456789012345678901234567890123456789012
c	LAST MODIFICATION:						
c	- May 7, 2002		 					
c #####################################################################
      subroutine  wave_forcing

c   Calculate the steady wave-induced-forcing for the vertical 
c   profiles: the terms designated as beta_{alpha} in FS97/4, 
c   eqs.(3),(22),(23).
c   We consider: WAVE FORCING ONLY UP TO THE INITIAL SHORELINE !!!
c   This means: even with the moving shoreline, the forcing is 
c   calculated only for the initally wet points; No corrections are done.
c
c   Calculate also the steady-streaming bottom shear stress: tauss
c   as in FS97/5, eq.(16)
c
c######################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'
      include 'pass.h'
c--- Local variables -------------------------------------------------
      integer  ncode, nxflag

      real*8 h2oh(nxm,nym), h2oh_x(nxm,nym), h2oh_y(nxm,nym),
     . theta_x(nxm,nym), theta_y(nxm,nym), aux1,
     . uwduwdx(nxm,nym), vwduwdy(nxm,nym), dw2dx(nxm,nym),
     . vwdvwdy(nxm,nym), uwdvwdx(nxm,nym), dw2dy(nxm,nym)

c=====================================================================
c      write(*,903)' Steady wave-induced-forcing for the vertical prof.'



      do j=1,ny
        do i=1,nxa(j)-1

c ## calculate short-wave forcing, eqn. (18)-(23) note FS97/4:


c          shwave_force_x(i,j) = Pass_Wave_Fx(i,j)
          shwave_force_x(i,j) = 0.

c          shwave_force_y(i,j) =  Pass_Wave_Fy(i,j)
          shwave_force_y(i,j) =  0.

c-- include steady-streaming in the U,V profiles
          tauss_x(i,j)= dsqrt(0.08d0/8.d0)*fcwij(i,j)*H(i,j)/ht(i,j)
     .                 *(u0(i,j)*costh(i,j))**2.d0
          tauss_y(i,j)= dsqrt(0.08d0/8.d0)*fcwij(i,j)*H(i,j)/ht(i,j)
     .                 *(u0(i,j)*sinth(i,j))**2.d0


        end do
        do i=nxa(j), nx
          shwave_force_x(i,j) = 0.d0
          shwave_force_y(i,j) = 0.d0
          tauss_x(i,j) = 0.d0
          tauss_y(i,j) = 0.d0
        end do
      end do  

c--- averaging of values -------------------------------
c      call average_3pty(nx,ny,vwduwdy,vwduwdy)
c      call average_3pty(nx,ny,vwdvwdy,vwdvwdy)
c      do i=1,nx
c         vwduwdy(i,1)=0.25d0*vwduwdy(i,2)+0.5d0*vwduwdy(i,1)
c     .               +0.25d0*vwduwdy(i,ny-1)
c         vwduwdy(i,ny)=vwduwdy(i,1)
c         vwdvwdy(i,1)=0.25d0*vwdvwdy(i,2)+0.5d0*vwdvwdy(i,1)
c     .               +0.25d0*vwdvwdy(i,ny-1)
c         vwdvwdy(i,ny)=vwdvwdy(i,1)
c      end do




903   format(/A)
      return 
      end


c #####################################################################
      subroutine  UV_profile(zeta,frict_x,frict_y,t,windx,windy,vs)
c     
c     calculates the velocity profiles for the general 3D disperion
c     as outlined in Kevin Note 12b eqs 46-67  (V1a^0)
c
c######################################################################
CDIR$ FLOW
      include 'winc_std_common.inc'

 
      real*8 force_xx, force_xy, force_yx, force_yy, wind_x, wind_y

c--- Local variables -------------------------------------------------
      integer i, j

      real*8 t, htot(nxm,nym), zeta(nxm,nym), 
     . frict_x(nxm,nym), frict_y(nxm,nym), 
     . f_x, f_y, inv_ht(nxm,nym), inv_vt(nxm,nym), aux1,
     . windx, windy,qwx_ht(nxm,nym),qwy_ht(nxm,nym),
     . e1xh(nxm,nym),vs(nxm,nym),
     . e1yh(nxm,nym)

c=====================================================================
      force_xx(i,j,t)= DTANH(t*delay)*dSxxdxn(i,j)
      force_xy(i,j,t)= DTANH(t*delay)*dSxydyn(i,j)
      force_yx(i,j,t)= DTANH(t*delay)*dSxydxn(i,j)
      force_yy(i,j,t)= DTANH(t*delay)*dSyydyn(i,j)
      wind_x(t) = DTANH(t*delay)*windx
      wind_y(t) = DTANH(t*delay)*windy

      aux1 = 1.d0/6.d0
      do j=1,ny
        do i=1,nxa(j)-2
	  htot(i,j) = ht(i,j)+zeta(i,j)
          inv_ht(i,j) = 1.d0/htot(i,j)
          qwx_ht(i,j)  = qwx(i,j)*inv_ht(i,j)
          qwy_ht(i,j)  = qwy(i,j)*inv_ht(i,j)
          inv_vt(i,j) = 1.d0/(v_t(i,j)+vs(i,j))

c ## assemble f_x as in eq.(46), note Kevin 12.b:
          f_x = (
     .    force_xx(i,j,t)
     .    +force_xy(i,j,t)
     .     +frict_x(i,j)-tauss_x(i,j)+wind_x(t)) *inv_ht(i,j)
     .     - shwave_force_x(i,j)

c ## calculate U1 coefficients: U1=d1x*z**2+e1x*z+f1x+f2x; 

          e1x(i,j)=(frict_x(i,j)-tauss_x(i,j))*inv_vt(i,j)
          e1xh(i,j)=e1x(i,j)*htot(i,j)
          d1x(i,j)=-.5d0*inv_vt(i,j)*(f_x)
          f1x(i,j)= -qwx(i,j)*inv_ht(i,j)-htot(i,j)*.5d0*e1x(i,j)
          fs1x(i,j)= -htot(i,j)*.5d0*e1x(i,j)
          f2x(i,j)=inv_vt(i,j)*(aux1*f_x*htot(i,j)*htot(i,j))
     
c ## assemble f_y as in eq.(47), note Kevin 12.3:
          f_y = +(force_yx(i,j,t)+force_yy(i,j,t) 
     .     +frict_y(i,j)-tauss_y(i,j)+wind_y(t)) *inv_ht(i,j)
     .     - shwave_force_y(i,j)


c ## calculate V1 coefficients: V1=d1y*z**2+e1y*z+f1y+f2y; 
          e1y(i,j)=(frict_y(i,j)-tauss_y(i,j))*inv_vt(i,j)
          e1yh(i,j)=e1y(i,j)*htot(i,j)
          d1y(i,j)=-.5d0*inv_vt(i,j)*(f_y)
          f1y(i,j)= -qwy(i,j)*inv_ht(i,j)-ht(i,j)*.5d0*e1y(i,j)
          fs1y(i,j)= -ht(i,j)*.5d0*e1y(i,j)
          f2y(i,j)=inv_vt(i,j)*(aux1*f_y*htot(i,j)*htot(i,j))
        end do
        do i=nxa(j)-1, nx
          e1x(i,j) = 0.d0
          d1x(i,j) = 0.d0
          f1x(i,j) = 0.d0
          fs1x(i,j) = 0.d0
          f2x(i,j) = 0.d0
          e1y(i,j) = 0.d0
          d1y(i,j) = 0.d0
          f1y(i,j) = 0.d0
          fs1y(i,j) = 0.d0
          f2y(i,j) = 0.d0
        end do
      end do  

994   format( 1x, 19(f13.8,1x))

      return 
      end


c #####################################################################

c #####################################################################
         subroutine coefcalc(Dxx,Dyy,Dxy,zeta,Mxx,Myy,Mxy,
     .     Bxx,Bxy,Byy,Axxx,Axxy,Axyx,Axyy,Ayyx,Ayyy,vs,Uo,Vo)
c
c     calculates the coefficients used in the generalized 3D dispersion
c     Currently utilizes assumptions from Ap's version only
c     
c     
c     Kevin Haas 5/99
c #####################################################################

      include 'winc_std_common.inc'
      
      integer i, j

      real*8 htot(nxm,nym),ht2(nxm,nym),ht3(nxm,nym),ht4(nxm,nym),
     . inv_vt(nxm,nym),inv_ht(nxm,nym),Dxx(nxm,nym),Dyy(nxm,nym),
     . Bxx(nxm,nym),Byy(nxm,nym),Bxy(nxm,nym),Uo(nxm,nym),Vo(nxm,nym), 
     . Dxy(nxm,nym),zeta(nxm,nym),Mxx(nxm,nym),Myy(nxm,nym),Mxy(nxm,nym)
     . ,Mxxap(nxm,nym),Mxyap(nxm,nym),Myyap(nxm,nym),aux1,
     . Lxx1(nxm,nym),Lxx2(nxm,nym),Lxx3(nxm,nym),
     . Lxy1(nxm,nym),Lxy2(nxm,nym),Lxy3(nxm,nym),
     . Lyy1(nxm,nym),Lyy2(nxm,nym),Lyy3(nxm,nym),
     . Lyx1(nxm,nym),Lyx2(nxm,nym),Lyx3(nxm,nym),
     . de1xdx(nxm,nym),de1xdy(nxm,nym),de1ydx(nxm,nym),de1ydy(nxm,nym),
     . dd1xdx(nxm,nym),dd1xdy(nxm,nym),dd1ydx(nxm,nym),dd1ydy(nxm,nym),
     . dfsxdx(nxm,nym),dfsxdy(nxm,nym),dfsydx(nxm,nym),dfsydy(nxm,nym),
     . fsumx(nxm,nym),fsumy(nxm,nym),vs(nxm,nym),
     . Axxx(nxm,nym),Axxy(nxm,nym),Axyx(nxm,nym),Axyy(nxm,nym),
     . Ayyx(nxm,nym),Ayyy(nxm,nym),dUo_dx(nxm,nym),dVo_dy(nxm,nym),
     . dUo_dy(nxm,nym),dVo_dx(nxm,nym),wstar

      aux1=1.d0/6.d0

      
      do j=1,ny
        do i=1,nxa(j)
	  htot(i,j) = ht(i,j)+zeta(i,j)
          ht2(i,j)  = htot(i,j)*htot(i,j)
          ht3(i,j)  = ht2(i,j)*htot(i,j)
          ht4(i,j)  = ht3(i,j)*htot(i,j)
          inv_ht(i,j) = 1.d0/htot(i,j)
          inv_vt(i,j) = 1.d0/(v_t(i,j)+vs(i,j))

           fsumx(i,j)=f1x(i,j)+f2x(i,j)
           fsumy(i,j)=f1y(i,j)+f2y(i,j)

        
      Dxy(i,j) = ht4(i,j)/63.d0*d1x(i,j)*d1y(i,j)+ht3(i,j)/36.d0*
     .         (d1x(i,j)*e1y(i,j)+d1y(i,j)*e1x(i,j))
     .         +ht2(i,j)/15.d0*(d1x(i,j)*(f1y(i,j)+f2y(i,j))+
     .         d1y(i,j)*(f1x(i,j)+f2x(i,j))+.75*e1x(i,j)*e1y(i,j))
     .         +htot(i,j)*0.125d0*(e1x(i,j)*(f1y(i,j)+f2y(i,j))+
     .         e1y(i,j)*(f1x(i,j)+f2x(i,j)))
     .         +1.d0/3.d0*(f1x(i,j)+f2x(i,j))*(f1y(i,j)+f2y(i,j))

      Dxy(i,j) = Dxy(i,j)*ht2(i,j)*inv_vt(i,j)

      Dyy(i,j) = ht4(i,j)/63.d0*d1y(i,j)*d1y(i,j)+ht3(i,j)/36.d0*
     .         (d1y(i,j)*e1y(i,j)+d1y(i,j)*e1y(i,j))
     .         +ht2(i,j)/15.d0*(d1y(i,j)*(f1y(i,j)+f2y(i,j))+
     .         d1y(i,j)*(f1y(i,j)+f2y(i,j))+.75*e1y(i,j)*e1y(i,j))
     .         +htot(i,j)*0.125d0*(e1y(i,j)*(f1y(i,j)+f2y(i,j))+
     .         e1y(i,j)*(f1y(i,j)+f2y(i,j)))
     .         +1.d0/3.d0*(f1y(i,j)+f2y(i,j))*(f1y(i,j)+f2y(i,j))

      Dyy(i,j) = Dyy(i,j)*ht2(i,j)*inv_vt(i,j)

      Dxx(i,j) = ht4(i,j)/63.d0*d1x(i,j)*d1x(i,j)+ht3(i,j)/36.d0*
     .         (d1x(i,j)*e1x(i,j)+d1x(i,j)*e1x(i,j))
     .         +ht2(i,j)/15.d0*(d1x(i,j)*(f1x(i,j)+f2x(i,j))+
     .         d1x(i,j)*(f1x(i,j)+f2x(i,j))+.75*e1x(i,j)*e1x(i,j))
     .         +htot(i,j)*0.125d0*(e1x(i,j)*(f1x(i,j)+f2x(i,j))+
     .         e1x(i,j)*(f1x(i,j)+f2x(i,j)))
     .         +1.d0/3.d0*(f1x(i,j)+f2x(i,j))*(f1x(i,j)+f2x(i,j))

      Dxx(i,j) = Dxx(i,j)*ht2(i,j)*inv_vt(i,j)

      Bxx(i,j)=(4.d0/63.d0*d1x(i,j)*d1x(i,j)*ht3(i,j)
     .     +aux1*e1x(i,j)*d1x(i,j)*ht2(i,j)
     .     +(4.d0/15.d0*d1x(i,j)*(f1x(i,j)+f2x(i,j))+.1d0
     .     *e1x(i,j)*e1x(i,j))*htot(i,j)
     .     +e1x(i,j)*(f1x(i,j)+f2x(i,j))*.25d0)

      Bxx(i,j) = -Bxx(i,j)*ht3(i,j)*inv_vt(i,j)


      Bxy(i,j)=(4.d0/63.d0*d1x(i,j)*d1y(i,j)*ht3(i,j)
     .     +aux1*.5d0*(e1x(i,j)*d1y(i,j)+e1y(i,j)*d1x(i,j))*ht2(i,j)
     .     +(2.d0/15.d0*(d1x(i,j)*(f1y(i,j)+f2y(i,j))
     .     +d1y(i,j)*(f1x(i,j)+f2x(i,j)))
     .     +.1d0*e1x(i,j)*e1y(i,j))*htot(i,j)
     .     +(e1x(i,j)*(f1y(i,j)+f2y(i,j))
     .     +e1y(i,j)*(f1x(i,j)+f2x(i,j)))*.125d0)

      Bxy(i,j) = -Bxy(i,j)*ht3(i,j)*inv_vt(i,j)


      Byy(i,j)=(4.d0/63.d0*d1y(i,j)*d1y(i,j)*ht3(i,j)
     .     +aux1*e1y(i,j)*d1y(i,j)*ht2(i,j)
     .     +(4.d0/15.d0*d1y(i,j)*(f1y(i,j)+f2y(i,j))+.1d0
     .     *e1y(i,j)*e1y(i,j))*htot(i,j)
     .     +e1y(i,j)*(f1y(i,j)+f2y(i,j))*.25d0)

      Byy(i,j) = -Byy(i,j)*ht3(i,j)*inv_vt(i,j)

ckevin 8/9/2000  Aproximates V1(h)Qw with -QwQw in M

      Mxxap(i,j)=d1x(i,j)*d1x(i,j)*ht4(i,j)*htot(i,j)*.2d0
     .     +(d1x(i,j)*e1x(i,j)*.5d0*ht4(i,j))
     .     +(d1x(i,j)*(fs1x(i,j)+f2x(i,j))*2.d0+e1x(i,j)*e1x(i,j))/3.d0
     .         *ht3(i,j)
     .     +(e1x(i,j)*(fs1x(i,j)+f2x(i,j))
c     .     +2.d0*qwx(i,j)*d1x(i,j)
     .         )*ht2(i,j)
     .     +2.d0*((fs1x(i,j)+f2x(i,j))*(fs1x(i,j)+f2x(i,j))*.5d0
c     .         +qwx(i,j)*e1x(i,j)
     .     )*htot(i,j)
     .     +(qwx(i,j)*inv_ht(i,j))*qwx(i,j)*2.d0
c     .     +(fs1x(i,j)+f2x(i,j))*qwx(i,j)*2.d0

c      Mxxap(i,j)=d1x(i,j)*d1x(i,j)*ht4(i,j)*htot(i,j)*.2d0
c     .     +(d1x(i,j)*e1x(i,j)*.5d0*ht4(i,j))
c     .     +(d1x(i,j)*(f1x(i,j)+f2x(i,j))*2.d0+e1x(i,j)*e1x(i,j))/3.d0
c     .         *ht3(i,j)
c     .     +(e1x(i,j)*(f1x(i,j)+f2x(i,j)))
c     .         *ht2(i,j)
c     .     +2.d0*((f1x(i,j)+f2x(i,j))*(f1x(i,j)+f2x(i,j))*.5d0
c     .         )*htot(i,j)
c     .     +(-qwx(i,j)*inv_ht(i,j))*qwx(i,j)*2.d0

      Myyap(i,j)=d1y(i,j)*d1y(i,j)*ht4(i,j)*htot(i,j)*.2d0
     .     +(d1y(i,j)*e1y(i,j)*.5d0*ht4(i,j))
     .     +(d1y(i,j)*(fs1y(i,j)+f2y(i,j))*2.d0+e1y(i,j)*e1y(i,j))/3.d0
     .         *ht3(i,j)
     .     +(e1y(i,j)*(fs1y(i,j)+f2y(i,j))
c     .     +2.d0*qwy(i,j)*d1y(i,j)
     .         )*ht2(i,j)
     .     +2.d0*((fs1y(i,j)+f2y(i,j))*(fs1y(i,j)+f2y(i,j))*.5d0
c     .         +qwy(i,j)*e1y(i,j)
     .     )*htot(i,j)
c     .     +(fs1y(i,j)+f2y(i,j))*qwy(i,j)*2.d0
     .     +(qwy(i,j)*inv_ht(i,j))*qwy(i,j)*2.d0

c           Myyap(i,j)=d1y(i,j)*d1y(i,j)*ht4(i,j)*htot(i,j)*.2d0
c     .     +(d1y(i,j)*e1y(i,j)*.5d0*ht4(i,j))
c     .     +(d1y(i,j)*(f1y(i,j)+f2y(i,j))*2.d0+e1y(i,j)*e1y(i,j))/3.d0
c     .         *ht3(i,j)
c     .     +(e1y(i,j)*(f1y(i,j)+f2y(i,j)))
c     .         *ht2(i,j)
c     .     +2.d0*((f1y(i,j)+f2y(i,j))*(f1y(i,j)+f2y(i,j))*.5d0
c     .         )*htot(i,j)
c     .     +(-qwy(i,j)*inv_ht(i,j))*qwy(i,j)*2.d0

      Mxyap(i,j)=d1x(i,j)*d1y(i,j)*ht4(i,j)*htot(i,j)*.2d0
     .     +(d1x(i,j)*e1y(i,j)+d1y(i,j)*e1x(i,j))*.25d0*ht4(i,j)
     .     +(d1x(i,j)*(fs1y(i,j)+f2y(i,j))+d1y(i,j)*(fs1x(i,j)
     .         +f2x(i,j))+e1x(i,j)*e1y(i,j))*aux1*2.d0*ht3(i,j)
     .     +(e1x(i,j)*(fs1y(i,j)+f2y(i,j))+e1y(i,j)*(fs1x(i,j)+
     .         f2x(i,j))
c     .     +2.d0*qwx(i,j)*d1y(i,j)+2.d0*qwy(i,j)*d1x(i,j)
     .     )*0.5d0*ht2(i,j)
     .     +((fs1x(i,j)+f2x(i,j))*(fs1y(i,j)+f2y(i,j))
c     .         +qwx(i,j)*e1y(i,j)+qwy(i,j)*e1x(i,j)
     .     )*htot(i,j)
c     .     +(fs1y(i,j)+f2y(i,j))*qwx(i,j)+(fs1x(i,j)+f2x(i,j))*
c     .         qwy(i,j)
     .     +(qwx(i,j)*inv_ht(i,j))*qwy(i,j)
     .     +(qwy(i,j)*inv_ht(i,j))*qwx(i,j)

c      Mxyap(i,j)=d1x(i,j)*d1y(i,j)*ht4(i,j)*htot(i,j)*.2d0
c     .     +(d1x(i,j)*e1y(i,j)+d1y(i,j)*e1x(i,j))*.25d0*ht4(i,j)
c     .     +(d1x(i,j)*(f1y(i,j)+f2y(i,j))+d1y(i,j)*(f1x(i,j)+
c     .         +f2x(i,j))+e1x(i,j)*e1y(i,j))*aux1*2.d0*ht3(i,j)
c     .     +(e1x(i,j)*(f1y(i,j)+f2y(i,j))+e1y(i,j)*(f1x(i,j)+
c     .         f2x(i,j))
c     .         )*0.5d0*ht2(i,j)
c     .     +((f1x(i,j)+f2x(i,j))*(f1y(i,j)+f2y(i,j))
c     .         )*htot(i,j)
c     .     +(-qwy(i,j)*inv_ht(i,j))*qwx(i,j)+(-qwx(i,j)*inv_ht(i,j))*
c     .         qwy(i,j)


         end do
      end do

     

      call deriv(d1x, d1y, e1x, dd1xdx, dd1xdy, dd1ydx,
     .		 de1xdy, 3, 1)
      call deriv(e1y, e1x, d1y, de1ydx, de1ydy, de1xdx,
     .		 dd1ydy, 3, 1)
      call deriv(fsumx, fsumy, fsumy, dfsxdx, dfsxdy, dfsydx,
     .		 dfsydy, 3, 1)
      call deriv(Uo, Vo, Vo, dUo_dx, dUo_dy, dVo_dx, dVo_dy,3,1)


      do j=1,ny
      do i=1,nxa(j)

      

      Lxx3(i,j)=dd1xdx(i,j)
      Lxy3(i,j)=dd1xdy(i,j)
      Lyx3(i,j)=dd1ydx(i,j)
      Lyy3(i,j)=dd1ydy(i,j)
      Lxx2(i,j)=de1xdx(i,j)
      Lxy2(i,j)=de1xdy(i,j)
      Lyx2(i,j)=de1ydx(i,j)
      Lyy2(i,j)=de1ydy(i,j)
      Lxx1(i,j)=dfsxdx(i,j)
      Lxy1(i,j)=dfsxdy(i,j)
      Lyx1(i,j)=dfsydx(i,j)
      Lyy1(i,j)=dfsydy(i,j)


      Mxx(i,j)=Mxxap(i,j)
      Mxy(i,j)=Mxyap(i,j)
      Myy(i,j)=Myyap(i,j)

      
      Axxx(i,j)=(d1x(i,j)*Lxx3(i,j)*2.d0/63.d0*ht4(i,j)
     .     +(d1x(i,j)*Lxx2(i,j)+e1x(i,j)*Lxx3(i,j))/18.d0*ht3(i,j)
     .     +((d1x(i,j)*Lxx1(i,j)+fsumx(i,j)*Lxx3(i,j))*2.d0/15.d0
     .          +e1x(i,j)*Lxx2(i,j)*.1d0)*ht2(i,j)
     .     +(e1x(i,j)*Lxx1(i,j)+fsumx(i,j)*Lxx2(i,j))*htot(i,j)*.25d0
     .     +(fsumx(i,j)*Lxx1(i,j))*aux1*4.d0)
      Axxx(i,j)=-Axxx(i,j)*inv_vt(i,j)*ht3(i,j)

      Axxy(i,j)=(d1x(i,j)*Lxy3(i,j)*2.d0/63.d0*ht4(i,j)
     .     +(d1x(i,j)*Lxy2(i,j)+e1x(i,j)*Lxy3(i,j))/18.d0*ht3(i,j)
     .     +((d1x(i,j)*Lxy1(i,j)+fsumx(i,j)*Lxy3(i,j))*2.d0/15.d0
     .          +e1x(i,j)*Lxy2(i,j)*.1d0)*ht2(i,j)
     .     +(e1x(i,j)*Lxy1(i,j)+fsumx(i,j)*Lxy2(i,j))*htot(i,j)*.25d0
     .     +(fsumx(i,j)*Lxy1(i,j))*aux1*4.d0)
      Axxy(i,j)=-Axxy(i,j)*inv_vt(i,j)*ht3(i,j)

      Axyx(i,j)=((d1y(i,j)*Lxx3(i,j)+d1x(i,j)*Lyx3(i,j))/63.d0*ht4(i,j)
     .     +(d1y(i,j)*Lxx2(i,j)+d1x(i,j)*Lyx2(i,j)+e1y(i,j)*Lxx3(i,j)
     .         +e1x(i,j)*Lyx3(i,j))/36.d0*ht3(i,j)
     .     +((d1y(i,j)*Lxx1(i,j)+d1x(i,j)*Lyx1(i,j)
     .         +fsumy(i,j)*Lxx3(i,j)+fsumx(i,j)*Lyx3(i,j))/15.d0
     .         +(e1y(i,j)*Lxx2(i,j)+e1x(i,j)*Lyx2(i,j))*.05d0)*ht2(i,j)
     .     +(e1y(i,j)*Lxx1(i,j)+e1y(i,j)*Lyx1(i,j)+fsumy(i,j)*Lxx2(i,j)
     .         +fsumx(i,j)*Lyx2(i,j))*htot(i,j)*.125d0
     .     +(fsumy(i,j)*Lxx1(i,j)+fsumx(i,j)*Lyx1(i,j))*aux1*2.d0)
      Axyx(i,j)=-Axyx(i,j)*inv_vt(i,j)*ht3(i,j)

      Axyy(i,j)=((d1y(i,j)*Lxy3(i,j)+d1x(i,j)*Lyy3(i,j))/63.d0*ht4(i,j)
     .     +(d1y(i,j)*Lxy2(i,j)+d1x(i,j)*Lyy2(i,j)+e1y(i,j)*Lxy3(i,j)
     .         +e1x(i,j)*Lyy3(i,j))/36.d0*ht3(i,j)
     .     +((d1y(i,j)*Lxy1(i,j)+d1x(i,j)*Lyy1(i,j)
     .         +fsumy(i,j)*Lxy3(i,j)+fsumx(i,j)*Lyy3(i,j))/15.d0
     .         +(e1y(i,j)*Lxy2(i,j)+e1x(i,j)*Lyy2(i,j))*.05d0)*ht2(i,j)
     .     +(e1y(i,j)*Lxy1(i,j)+e1y(i,j)*Lyy1(i,j)+fsumy(i,j)*Lxy2(i,j)
     .         +fsumx(i,j)*Lyy2(i,j))*htot(i,j)*.125d0
     .     +(fsumy(i,j)*Lxy1(i,j)+fsumx(i,j)*Lyy1(i,j))*aux1*2.d0)
      Axyy(i,j)=-Axyy(i,j)*inv_vt(i,j)*ht3(i,j)


      Ayyx(i,j)=(d1y(i,j)*Lyx3(i,j)*2.d0/63.d0*ht4(i,j)
     .     +(d1y(i,j)*Lyx2(i,j)+e1y(i,j)*Lyx3(i,j))/18.d0*ht3(i,j)
     .     +((d1y(i,j)*Lyx1(i,j)+fsumy(i,j)*Lyx3(i,j))*2.d0/15.d0
     .          +e1y(i,j)*Lyx2(i,j)*.1d0)*ht2(i,j)
     .     +(e1y(i,j)*Lyx1(i,j)+fsumy(i,j)*Lyx2(i,j))*htot(i,j)*.25d0
     .     +(fsumy(i,j)*Lyx1(i,j))*aux1*4.d0)
      Ayyx(i,j)=-Ayyx(i,j)*inv_vt(i,j)*ht3(i,j)

      Ayyy(i,j)=(d1y(i,j)*Lyy3(i,j)*2.d0/63.d0*ht4(i,j)
     .     +(d1y(i,j)*Lyy2(i,j)+e1y(i,j)*Lyy3(i,j))/18.d0*ht3(i,j)
     .     +((d1y(i,j)*Lyy1(i,j)+fsumy(i,j)*Lyy3(i,j))*2.d0/15.d0
     .          +e1y(i,j)*Lyy2(i,j)*.1d0)*ht2(i,j)
     .     +(e1y(i,j)*Lyy1(i,j)+fsumy(i,j)*Lyy2(i,j))*htot(i,j)*.25d0
     .     +(fsumy(i,j)*Lyy1(i,j))*aux1*4.d0)
      Ayyy(i,j)=-Ayyy(i,j)*inv_vt(i,j)*ht3(i,j)


c     calculate Vd coefficients

        uda4(i,j)=(Uo(i,j)*dd1xdx(i,j)+Vo(i,j)*dd1xdy(i,j))*aux1*0.5d0
        uda3(i,j)=(Uo(i,j)*(2.d0*d1x(i,j)*dhodxn(i,j)+de1xdx(i,j))
     .       +Vo(i,j)*(2.d0*d1x(i,j)*dhodyn(i,j)+de1xdy(i,j)))*aux1
        uda2(i,j)=(Uo(i,j)*(e1x(i,j)*dhodxn(i,j)+dfsxdx(i,j))
     .       +Vo(i,j)*(e1x(i,j)*dhodyn(i,j)+dfsxdy(i,j)))*.5d0

        vda4(i,j)=(Uo(i,j)*dd1ydx(i,j)+Vo(i,j)*dd1ydy(i,j))*aux1*0.5d0
        vda3(i,j)=(Uo(i,j)*(2.d0*d1y(i,j)*dhodxn(i,j)+de1ydx(i,j))
     .       +Vo(i,j)*(2.d0*d1y(i,j)*dhodyn(i,j)+de1ydy(i,j)))*aux1
        vda2(i,j)=(Uo(i,j)*(e1y(i,j)*dhodxn(i,j)+dfsydx(i,j))
     .       +Vo(i,j)*(e1y(i,j)*dhodyn(i,j)+dfsydy(i,j)))*.5d0

        udb4(i,j)=(dUo_dx(i,j)*d1x(i,j)+dUo_dy(i,j)*d1y(i,j))*aux1*0.5d0
        udb3(i,j)=(dUo_dx(i,j)*e1x(i,j)+dUo_dy(i,j)*e1y(i,j))*aux1
        udb2(i,j)=(dUo_dx(i,j)*fsumx(i,j)+dUo_dy(i,j)*fsumy(i,j))*0.5d0
        vdb4(i,j)=(dVo_dx(i,j)*d1x(i,j)+dVo_dy(i,j)*d1y(i,j))*aux1*0.5d0
        vdb3(i,j)=(dVo_dx(i,j)*e1x(i,j)+dVo_dy(i,j)*e1y(i,j))*aux1
        vdb2(i,j)=(dVo_dx(i,j)*fsumx(i,j)+dVo_dy(i,j)*fsumy(i,j))*0.5d0

        Wstar=(Uo(i,j))*dhodxn(i,j)
     .       +(Vo(i,j))*dhodyn(i,j)

        udw4(i,j)=-d1x(i,j)*(dUo_dx(i,j)+dVo_dy(i,j))*aux1
        udw3(i,j)=-e1x(i,j)*(dUo_dx(i,j)+dVo_dy(i,j))*aux1
     .       -wstar*d1x(i,j)*aux1*2.d0
        udw2(i,j)=-e1x(i,j)*wstar*.5d0

        vdw4(i,j)=-d1y(i,j)*(dUo_dx(i,j)+dVo_dy(i,j))*aux1
        vdw3(i,j)=-e1y(i,j)*(dUo_dx(i,j)+dVo_dy(i,j))*aux1
     .       -wstar*d1y(i,j)*aux1*2.d0
        vdw2(i,j)=-e1y(i,j)*wstar*.5d0

        udc(i,j)=-(uda4(i,j)+udb4(i,j)+udw4(i,j))*ht4(i,j)*.2d0
     .     -(uda3(i,j)+udb3(i,j)+udw3(i,j))*ht3(i,j)*.25d0
     .     -(uda2(i,j)+udb2(i,j)+udw2(i,j))*ht2(i,j)*aux1*2.d0

        vdc(i,j)=-(vda4(i,j)+vdb4(i,j)+vdw4(i,j))*ht4(i,j)*.2d0
     .     -(vda3(i,j)+vdb3(i,j)+vdw3(i,j))*ht3(i,j)*.25d0
     .     -(vda2(i,j)+vdb2(i,j)+vdw2(i,j))*ht2(i,j)*aux1*2.d0

          uda4(i,j)=uda4(i,j)*inv_vt(i,j)
          uda3(i,j)=uda3(i,j)*inv_vt(i,j)
          uda2(i,j)=uda2(i,j)*inv_vt(i,j)
          vda4(i,j)=vda4(i,j)*inv_vt(i,j)
          vda3(i,j)=vda3(i,j)*inv_vt(i,j)
          vda2(i,j)=vda2(i,j)*inv_vt(i,j)
          udb4(i,j)=udb4(i,j)*inv_vt(i,j)
          udb3(i,j)=udb3(i,j)*inv_vt(i,j)
          udb2(i,j)=udb2(i,j)*inv_vt(i,j)
          vdb4(i,j)=vdb4(i,j)*inv_vt(i,j)
          vdb3(i,j)=vdb3(i,j)*inv_vt(i,j)
          vdb2(i,j)=vdb2(i,j)*inv_vt(i,j)
          udw4(i,j)=udw4(i,j)*inv_vt(i,j)
          udw3(i,j)=udw3(i,j)*inv_vt(i,j)
          udw2(i,j)=udw2(i,j)*inv_vt(i,j)
          vdw4(i,j)=vdw4(i,j)*inv_vt(i,j)
          vdw3(i,j)=vdw3(i,j)*inv_vt(i,j)
          vdw2(i,j)=vdw2(i,j)*inv_vt(i,j)
          udc(i,j)=udc(i,j)*inv_vt(i,j)
          vdc(i,j)=vdc(i,j)*inv_vt(i,j)
      end do

        do i=nxa(j)+1, nx
          Dxx(i,j) = 0.d0
          Dyy(i,j) = 0.d0
          Dxy(i,j) = 0.d0
          Mxx(i,j) = 0.d0
          Mxy(i,j) = 0.d0
          Myy(i,j) = 0.d0
          Bxx(i,j) = 0.d0
          Bxy(i,j) = 0.d0
          Byy(i,j) = 0.d0          
          uda4(i,j)= 0.d0      
          uda3(i,j)= 0.d0      
          uda2(i,j)= 0.d0      
          vda4(i,j)= 0.d0      
          vda3(i,j)= 0.d0      
          vda2(i,j)= 0.d0      
          udb4(i,j)= 0.d0      
          udb3(i,j)= 0.d0      
          udb2(i,j)= 0.d0      
          vdb4(i,j)= 0.d0      
          vdb3(i,j)= 0.d0      
          vdb2(i,j)= 0.d0      
          udw4(i,j)= 0.d0      
          udw3(i,j)= 0.d0      
          udw2(i,j)= 0.d0      
          vdw4(i,j)= 0.d0      
          vdw3(i,j)= 0.d0      
          vdw2(i,j)= 0.d0      
          udc(i,j)= 0.d0      
          vdc(i,j)= 0.d0      
        end do
       end do  

905   format(500(f12.8))
         return
         end
  







